#ifndef ACM_H_
#define ACM_H_

#include "stdint.h"
// this is the program mode: SIMULATION does not involve CAN blocks; 
// while REAL mode will interact with the CAN blocks and can be used
// on real cars.
#define SIMULATION
// #define REAL

#define ECAN_BUFFER_COUNT 4
#define ENABLE_MODULE_ECAN_1
#define ENABLE_MODULE_OC_1
// The RX table contains this many mailboxes.
#define CAN_TABLE_RX_LENGTH 48	
// The TX table contains this many mailboxes.
#define CAN_TABLE_TX_LENGTH 24	
// This is the maximum number of CAN messages in each TTCAN cycle.
#define E15AER_TTCAN_SCHEDULE_WINDOWS 10
// This is the number of cycles within the TTCAN schedule.
#define E15AER_TTCAN_SCHEDULE_CYCLES 3
#define ACM_CAN_ADDRESS 0x460
// The master controller will be installed in one of four positions.  This
// definition is used in a switch/case when determining the column and row
// in the lookup table.
// 0 - Accelerometer positive X axis is pointed toward the front of the car
// 1 - Accelerometer positive X axis is pointed toward the right side of the car
// 2 - Accelerometer positive X axis is pointed toward the rear of the car
// 3 - Accelerometer positive X axis is pointed toward the left side of the car
#define ACM_DIRECTION_X 3
// This is the number of ADC inputs the ACM reads.
#define ACM_ADC_INPUT_COUNT 4
// This is the number of servos that the ACM will be controlling.
#define ACM_SERVO_COUNT 8

// These are the servo indexes.  Use them to reference servo information in
// the servo control middleware.  If these numbers are changed be sure to
// change acmSensorUpdateSwLimit() so that the bits in iSwLimit stay in the
// same order.

#define ACM_SERVO_LFL 0    // Left front lower
#define ACM_SERVO_LFU 1    // Left front upper

#define ACM_SERVO_RFL 2    // Right front lower
#define ACM_SERVO_RFU 3    // Right front upper

#define ACM_SERVO_LRL 4    // Left rear lower
#define ACM_SERVO_LRU 5    // Left rear upper

#define ACM_SERVO_RRL 6    // Right rear lower
#define ACM_SERVO_RRU 7    // Right rear upper

// These are the "servo enable" definitions.  Set a servo to 0 if it's broken
// or missing.  The application will ignore that servo and continue to operate
// other (enabled) servos normally.

#define ACM_SERVO_ENABLE_LFL   1
#define ACM_SERVO_ENABLE_LFU   1

#define ACM_SERVO_ENABLE_RFL   1
#define ACM_SERVO_ENABLE_RFU   1

#define ACM_SERVO_ENABLE_LRL   1
#define ACM_SERVO_ENABLE_LRU   1

#define ACM_SERVO_ENABLE_RRL   1
#define ACM_SERVO_ENABLE_RRU   1

// Sometimes the waveform may need to be inverted due to hardware issue
// associated with either the controller board or the servo.

#define ACM_SERVO_INVERT_LFL   0
#define ACM_SERVO_INVERT_LFU   0

#define ACM_SERVO_INVERT_RFL   0
#define ACM_SERVO_INVERT_RFU   0

#define ACM_SERVO_INVERT_LRL   0
#define ACM_SERVO_INVERT_LRU   0

#define ACM_SERVO_INVERT_RRL   0
#define ACM_SERVO_INVERT_RRU   0

// If a servo is stuck, a servo has failed, or if a limit switch is broken, the
// calibration routine will fail.  This is the maximum number of ticks (tick
// function calls) that a servo could need to finish one stage of calibration.
// If the servo is ticked past this number, the calibration routine will abort
// and use manually calibrated values.
#define ACM_CALIBRATE_TICK_LIMIT_MAX_LFL 165
#define ACM_CALIBRATE_TICK_LIMIT_MAX_LFU 250

#define ACM_CALIBRATE_TICK_LIMIT_MAX_RFL 165
#define ACM_CALIBRATE_TICK_LIMIT_MAX_RFU 250

#define ACM_CALIBRATE_TICK_LIMIT_MAX_LRL 250
#define ACM_CALIBRATE_TICK_LIMIT_MAX_LRU 350

#define ACM_CALIBRATE_TICK_LIMIT_MAX_RRL 250
#define ACM_CALIBRATE_TICK_LIMIT_MAX_RRU 350

// Likewise, if a servo calibrates in just a few ticks, that means that a limit
// switch could be stuck closed or something else has gone wrong.
#define ACM_CALIBRATE_TICK_LIMIT_MIN_LFL 16
#define ACM_CALIBRATE_TICK_LIMIT_MIN_LFU 16

#define ACM_CALIBRATE_TICK_LIMIT_MIN_RFL 16
#define ACM_CALIBRATE_TICK_LIMIT_MIN_RFU 16

#define ACM_CALIBRATE_TICK_LIMIT_MIN_LRL 16
#define ACM_CALIBRATE_TICK_LIMIT_MIN_LRU 16

#define ACM_CALIBRATE_TICK_LIMIT_MIN_RRL 16
#define ACM_CALIBRATE_TICK_LIMIT_MIN_RRU 16

// These numbers represent the min/max physical limits of the servos in terms
// of PWM timer count values.  Look at the ACM timer code to see how these
// numbers are caluclated.
#define ACM_SERVO_POSITION_MINIMUM 600
#define ACM_SERVO_POSITION_MAXIMUM 1270

#define ACM_SERVO_POWER_OFF_TICK_COUNT 10

// These are the bit offsets for the dashboard mode and override switches.
#define ACM_BIT_SW_MODE_1 0
#define ACM_BIT_SW_MODE_2 1
#define ACM_BIT_SW_MODE_3 2
#define ACM_BIT_SW_OVERRIDE 3

#define ACM_TABLE_COLUMN_MIN 0
#define ACM_TABLE_COLUMN_CENTER 20
#define ACM_TABLE_COLUMN_MAX 40

#define ACM_TABLE_ROW_MIN 0
#define ACM_TABLE_ROW_CENTER 13
#define ACM_TABLE_ROW_MAX 33
// These (absolute) values are defined in units of milli-gs.
#define ACM_TABLE_LIMIT_LATERAL 1900
#define ACM_TABLE_LIMIT_LONGITUDINAL_ACCEL 1200
#define ACM_TABLE_LIMIT_LONGITUDINAL_DECEL 1900

#define HEARTBEAT_MWARE_TIMER_COUNT 16

// These are the heartbeat monitor timer indexes.
#define ACM_HEARTBEAT_TIMER_ACW_LF 0
#define ACM_HEARTBEAT_TIMER_ACW_RF 1
#define ACM_HEARTBEAT_TIMER_ACW_LR 2
#define ACM_HEARTBEAT_TIMER_ACW_RR 3
// This is the heartbeat monitor timer index for the ACM.
#define ACW_HEARTBEAT_TIMER_ACM 0

// This value is a multiplier for the hardware timer period.  If the hardware
// timer is se to decrement the heartbeat values every 10 ms, a heartbeat value
// will time out after 10ms * ACM_HEARTBEAT_TIMEOUT_MULTIPLIER.
#define ACM_HEARTBEAT_TIMEOUT_MULTIPLIER 50
// This value is a multiplier for the hardware timer period.  If the hardware
// timer is se to decrement the heartbeat values every 10 ms, a heartbeat value
// will time out after 10ms * HEARTBEAT_TIMEOUT_MULTIPLIER.
#define ACW_HEARTBEAT_TIMEOUT_MULTIPLIER 50

// These are the wing board indexes.  I use them to distinguish between whole
// wing boards in the sensor scan functions.

#define ACM_WING_LEFT_FRONT    0
#define ACM_WING_RIGHT_FRONT   1
#define ACM_WING_LEFT_REAR     2
#define ACM_WING_RIGHT_REAR    3

/*
			macros for ACW
*/
#define ACW_LF_CAN_ADDRESS 0x462
#define ACW_RF_CAN_ADDRESS 0x464
#define ACW_LR_CAN_ADDRESS 0x466
#define ACW_RR_CAN_ADDRESS 0x468

// #define ACW_CAN_ADDRESS 0x462  // Left front wing
// #define ACW_CAN_ADDRESS 0x464  // Right front wing
// #define ACW_CAN_ADDRESS 0x466  // Left rear wing
// #define ACW_CAN_ADDRESS 0x468  // Right rear wing
#define ACW_SERVO_COUNT 2
// These are the servo indexes.
#define ACW_SERVO_LOWER 0
#define ACW_SERVO_UPPER 1

// static const uint8_t __attribute__ ((space(auto_psv))) aAeroLeftFront0[34][41] =
static const uint8_t aAeroLeftFront0[34][41] =
{
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0},
    {0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0},
    {0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0},
    {0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0}
};

// static const uint8_t __attribute__ ((space(auto_psv))) aAeroRightFront0[34][41] =
static const uint8_t aAeroRightFront0[34][41] =
{
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0},
    {0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0},
    {0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0},
    {0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0}
};

// static const uint8_t __attribute__ ((space(auto_psv))) aAeroLeftRear0[34][41] =
static const uint8_t aAeroLeftRear0[34][41] =
{
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0},
    {0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0},
    {0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0},
    {0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0}
};

// static const uint8_t __attribute__ ((space(auto_psv))) aAeroRightRear0[34][41] =
static const uint8_t aAeroRightRear0[34][41] =
{
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0,0},
    {0,0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0,0},
    {0,0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0,0},
    {0,0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0,0},
    {0,0,0,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,100,0,0,0}
};

typedef enum
{
    CanTable_Receive    =   0,
    CanTable_Transmit   =   1
} CanTable_Type_e;

// Enumeration for AA system state.
typedef enum
{
    ACM_State_Active0,
    ACM_State_Active1,
    ACM_State_Close,
    ACM_State_Demo,
    ACM_State_Lock,
    ACM_State_Manual,
    ACM_State_Open,
    ACM_State_Shutdown
} ACM_State_e;

// Enumeration for each quarter-wing.
typedef enum
{
    ACM_Wing_Left_Front = 0,
    ACM_Wing_Left_Rear = 1,
    ACM_Wing_Right_Front = 2,
    ACM_Wing_Right_Rear = 3
} ACM_Wing;


// Define ECANBUFFER
typedef uint16_t tECANBuffer [ECAN_BUFFER_COUNT][8];


// Define CAN table
typedef struct
{
    uint8_t bEid;       // True for extended ID (29b), false for standard (11b)

    // bEnabled is set for messages that are in use and cleared for messages
    // that are not used.  Applications are responsible for manipulating this
    // flag.  "Messages that are in use" implies that the message data structure
    // is available for transmission.
    uint8_t bEnabled;

    // bPending has different meanings depending on the context.  Applications
    // are responsible for manipulating this flag.

    // For received messages, bPending = 1 means that the message data struct
    // was populated by the CAN RX ISR, but the application has not yet had a
    // chance to read the data with a canInput() function.

    // For transmitted messages, the flag is "dead".  Setting or clearing
    // the variable has no effect, and the flag is not changed by any part
    // of the HAL or middleware.
    uint8_t bPending;
    
    // bReading is set for CAN messages that are currently being read.  The CAN
    // table middleware is responsible for manipulating this flag.
    
    // Assume a CAN frame arrives from the bus.  If an application is reading
    // data from the CAN table and that data is suddenly replaced, the resulting
    // sensor data could be corrupted.  When this flag is set, CAN frames will
    // bounce off and not be stored.
    uint8_t bReading;

    // bWriting is set for CAN messages that are currently being written.
    // Applications are responsible for manipulating this flag.

    // If a CAN TX interrupt fires while an application is in the middle of
    // updating the data fields in a CAN data struct, the frame will be
    // transmitted with garbage data.
    
    // Applications should set/clear this flag in their CAN output functions
    // and then check the flag when attempting to TX a CAN message.
    uint8_t bWriting;

    uint32_t iId;       // Message identifier, 11 or 29 bits
    uint8_t aData[8];   // Data bytes
    uint8_t iDlc;       // Data length code
    uint8_t iFilter;    // Index of the filter that matched this message

} CanMessage;

// Define TTCAN window object
typedef struct
{
    uint32_t * aMessageIds;
    uint16_t iNodeAddress;
} TtCanWindow;

// Define xyz coordinate of accelerometer
typedef struct
{
    int16_t iXAccel;
    int16_t iYAccel;
    int16_t iZAccel;
} XyzData;


// *****************************************************************************
// Datatypes, Definitions & Enumerations

// This is the aero sensor object data structure.

// aAdcData - raw data from the ADC is stored here.

// bDuplicate - set to 1 when a duplicate heartbeat is detected

// bShutdown - set to 1 when the system needs to shutdown for some reason

// iBatteryAdc - battery voltage measured by onboard ADC/voltage divider
// iBatteryCan - battery voltage reported by PE3 ECU (over CAN)

// iEngine Rpm - exactly what it says on the tin, comes from PE3 ECU

// iSwExt[0] - External switch, MCU_SW1
// iSwExt[1] - External switch, MCU_SW2
// iSwExt[2] - External switch, MCU_SW3
// iSwExt[3] - Steering override, MCU_SW4
// iSwExt[4] - reserved
// iSwExt[5] - reserved
// iSwExt[6] - reserved
// iSwExt[7] - reserved

// iSwLimit[0] - Left front lower switch
// iSwLimit[1] - Left front upper switch
// iSwLimit[2] - Right front lower switch
// iSwLimit[3] - Right front upper switch

// iSwLimit[4] - Left rear lower switch
// iSwLimit[5] - Left rear upper switch
// iSwLimit[6] - Right rear lower switch
// iSwLimit[7] - Right rear upper switch

// iWheelRps - the wheel speed, comes from PE3 ECU

// stXyzData - data from the accelerometer

typedef struct
{
    uint16_t aAdcData[ACM_ADC_INPUT_COUNT];

    uint8_t bDuplicateHb;
    uint8_t bShutdown;

    uint16_t iBatteryAdc;
    uint16_t iBatteryCan;

    uint16_t iEngineRpm;

    uint8_t iSwExt;
    uint8_t iSwLimit;

    uint16_t iWheelRps;

    XyzData stXyzData;
} AcmSensorData;

// bDuplicate - set to 1 when a duplicate heartbeat is detected

// bShutdown - set to 1 when the system needs to shutdown for some reason

// iPosLower - lower servo position read from CAN input
// iPosUpper - upper servo position read from CAN input

// iSwLimit[0] - Lower switch
// iSwLimit[1] - Upper switch

typedef struct
{
    uint8_t bDuplicateHb;
    uint8_t bShutdown;

    uint16_t iPosLower;
    uint16_t iPosUpper;

    uint8_t iSwLimit;

} AcwSensorData;

// This struct contains position and status information for one servo.
typedef struct
{
    // For use with an application.
    uint8_t bCalibrationFailed;

    // Set to false to disable the servo, in case the servo is broken or not
    // connected.
    uint8_t bEnabled;

    // Set to true to invert the PWM waveform.
    uint8_t bInvert;

    // This value is set to true when a servo is asked to move beyond either
    // the minimum or maximum position limit.
    uint8_t bLimitViolation;

    // Set to true when the servo is moving, false otherwise.
    uint8_t bMoving;

    // Set to true when the servo is powered.
    uint8_t bPowered;

    // All positions are measured in microseconds.  The duty cycle is equal to
    // the position divided by the PWM timer period value.  The timer period
    // value is set by the application within its timer initialization
    // functions.  Assuming the timer is set to interrupt every 4.0
    // milliseconds, a position value of 2000 would yield a 50% duty cycle.
    
    uint16_t iPosCurrent;   // Current position
    uint16_t iPosMax;       // Maximum position
    uint16_t iPosMin;       // Minimum position
    uint16_t iPosNeutral;   // Neutral position
    uint16_t iPosPrevious;  // The position before the current position
    uint16_t iPosTarget;    // The target position
    uint16_t iPosTemp;      // A temporary variable for application use

    // This is the servo unique identifier, for use by an application.
    uint8_t iServo;

    // This is the number of the times the servo has been ticked / moved.  This
    // value is for use by an application.
    uint32_t iTickCount;

    // Maximum tick count.  For use with an application.
    uint32_t iTickMax;

    // Minimum tick count.  For use with an application.
    uint32_t iTickMin;

    // Ticks since last movement.  For use with an application.
    uint32_t iTickSinceLastMove;

} ServoData;

//tECANBuffer aCAN1Buffer __attribute__((aligned(ECAN_BUFFER_COUNT*16)));
tECANBuffer aCAN1Buffer;					// aCAN1Buffer accept data from DMA
tECANBuffer aCAN1Buffer_acw_1;
tECANBuffer aCAN1Buffer_acw_2;
ACM_State_e eACMState = ACM_State_Active0;	//ACM_State_Active0		//ACM_State_Close
CanMessage aCanTableRx[CAN_TABLE_RX_LENGTH];
CanMessage aCanTableTx[CAN_TABLE_TX_LENGTH];
CanMessage aCanTableRx_acw_1[CAN_TABLE_RX_LENGTH];
CanMessage aCanTableRx_acw_2[CAN_TABLE_RX_LENGTH];
CanMessage aCanTableTx_acw_1[CAN_TABLE_TX_LENGTH];
CanMessage aCanTableTx_acw_2[CAN_TABLE_TX_LENGTH];
CanMessage stDeadMessage;
TtCanWindow aE15AerSchedule[E15AER_TTCAN_SCHEDULE_CYCLES][E15AER_TTCAN_SCHEDULE_WINDOWS];
AcmSensorData stAcmSensor;
AcwSensorData stAcwSensor_1;
AcwSensorData stAcwSensor_2;
ServoData aAcmServo[ACM_SERVO_COUNT];
ServoData aAcwServo_1[ACW_SERVO_COUNT];
ServoData aAcwServo_2[ACW_SERVO_COUNT];
uint8_t aAcmCalibrateState[ACM_SERVO_COUNT];
uint16_t aHeartbeatTimer[HEARTBEAT_MWARE_TIMER_COUNT];
uint16_t aHeartbeatTimer_acw_1[HEARTBEAT_MWARE_TIMER_COUNT];
uint16_t aHeartbeatTimer_acw_2[HEARTBEAT_MWARE_TIMER_COUNT];

uint32_t TIMER1_OUT[11];
uint32_t TIMER1_OUT_ACW_1[11];
uint32_t TIMER1_OUT_ACW_2[11];
uint16_t ACW_CAN_ADDRESS_1 = 0;
uint16_t ACW_CAN_ADDRESS_2 = 0;

uint32_t T3_OUT_acw_1[11];
uint32_t T3_OUT_acw_2[11];

// these two var are for acw
uint8_t iACWTTCan1Cycle_1 = 0;
uint8_t iACWTTCan1Cycle_2 = 0;
uint8_t iACWTTCan1Window_1 = 0;
uint8_t iACWTTCan1Window_2 = 0;
uint32_t id_acw_1 = 0;			// used in acw part for receiving id. for REAL mode
uint32_t id_acw_2 = 0;
uint32_t id_acm = 0;
uint8_t acwtimer_1 = 0;			// 1 is one, 0 is off
uint8_t acwtimer_2 = 0;

void canInitMessage(CanMessage * stCanMessage);
void can1AbortTx(int n);
void can1AbortTx_acw_1(int n);
void can1AbortTx_acw_2(int n);
void ctInit(void);
void ctSetMessage(CanMessage * stMessage, CanTable_Type_e eCanTable, uint16_t iTableIndex);
void ctSetMessage_acw_1(CanMessage * stMessage, CanTable_Type_e eCanTable, uint16_t iTableIndex);
void ctSetMessage_acw_2(CanMessage * stMessage, CanTable_Type_e eCanTable, uint16_t iTableIndex);
uint16_t ctGetMessageIndex(CanTable_Type_e eCanTable, uint32_t iCanId);
uint16_t ctGetMessageIndex_acw_1(CanTable_Type_e eCanTable, uint32_t iCanId);
uint16_t ctGetMessageIndex_acw_2(CanTable_Type_e eCanTable, uint32_t iCanId);
uint32_t e15aerTtCanGetId(uint16_t iNodeAddress, uint8_t iCycleIndex, uint8_t iWindowIndex);
CanMessage * ctGetMessageById(CanTable_Type_e eCanTable, uint32_t iCanId);
CanMessage * ctGetMessageById_acw_1(CanTable_Type_e eCanTable, uint32_t iCanId);
CanMessage * ctGetMessageById_acw_2(CanTable_Type_e eCanTable, uint32_t iCanId);
void can1RxMessage(uint8_t iBufferIndex, CanMessage * stCanMessage);
void can1RxMessage_acw_1(uint8_t iBufferIndex, CanMessage * stCanMessage);
void can1RxMessage_acw_2(uint8_t iBufferIndex, CanMessage * stCanMessage);
void directRxMessage(uint8_t iBufferIndex, CanMessage * stCanMessage);
void directRxMessage_acw_1(uint8_t iBufferIndex, CanMessage * stCanMessage);
void directRxMessage_acw_2(uint8_t iBufferIndex, CanMessage * stCanMessage);
void servo_mwSetPosition(ServoData * stServoData, uint16_t iPosition);
AcmSensorData * acmSensorRet(void);
ServoData * acmServoRet(void);
uint8_t * acmCalibrateRet(void);
ACM_State_e * acmStateRet(void);
uint16_t acwCanGetAddress_1(void);
uint16_t acwCanGetAddress_2(void);
uint8_t * acwCan1GetCycle_1(void);
uint8_t * acwCan1GetCycle_2(void);
uint8_t * acwCan1GetWindow_1(void);
uint8_t * acwCan1GetWindow_2(void);
ServoData * acwServoRet_1(void);
ServoData * acwServoRet_2(void);
AcwSensorData * acwSensorRet_1(void);
AcwSensorData * acwSensorRet_2(void);
void heartbeat_mwDecrement(uint8_t iTimer);
void heartbeat_mwDecrement_acw_1(uint8_t iTimer);
void heartbeat_mwDecrement_acw_2(uint8_t iTimer);
void heartbeat_mwSet(uint8_t iTimer, uint16_t iValue);
void heartbeat_mwSet_acw_1(uint8_t iTimer, uint16_t iValue);
void heartbeat_mwSet_acw_2(uint8_t iTimer, uint16_t iValue);
#endif



