// This is the CAN event interrupt handler for bus master nodes.  Master nodes
// do not respond to the 0x3200 or 0x3202 sync frame IDs.  The ACL is the bus
// master for the aero bus which is connected to its CAN1 module.

// The ACL transmits 0x3200 to synchronize the aero bus.

#include "acm.h"
#include "acmIsrC1.h"

uint8_t C1INTFbits_RBIF;
uint8_t C1RXFUL1bits_RXFUL1;       	
uint8_t C1RXFUL1bits_RXFUL2;
uint8_t C1RXFUL1bits_RXFUL3;

uint32_t CAN1_OUT[12];	//output matrix

/********************Local Functions****************/

void acmCanInitTableRx(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // Hearbeat - ACM
    canInitMessage(&stMessage);
    stMessage.iId = ACM_CAN_ADDRESS;

    ctSetMessage(&stMessage, CanTable_Receive, 0);

    // *************************************************************************
    // Heartbeat - AACW Left Front
    canInitMessage(&stMessage);
    stMessage.iId = 0x462;

    ctSetMessage(&stMessage, CanTable_Receive, 1);

    // *************************************************************************
    // Heartbeat - AACW Right Front
    canInitMessage(&stMessage);
    stMessage.iId = 0x464;

    ctSetMessage(&stMessage, CanTable_Receive, 2);

    // *************************************************************************
    // Heartbeat - AACW Left Rear
    canInitMessage(&stMessage);
    stMessage.iId = 0x466;

    ctSetMessage(&stMessage, CanTable_Receive, 3);

    // *************************************************************************
    // Heartbeat - AACW Right Rear
    canInitMessage(&stMessage);
    stMessage.iId = 0x468;

    ctSetMessage(&stMessage, CanTable_Receive, 4);

    // *************************************************************************
    // Active Aero Control Limit Switches - Left Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e00;

    ctSetMessage(&stMessage, CanTable_Receive, 5);

    // *************************************************************************
    // Active Aero Control Limit Switches - Right Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e02;

    ctSetMessage(&stMessage, CanTable_Receive, 6);

    // *************************************************************************
    // Active Aero Control Limit Switches - Left Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e04;

    ctSetMessage(&stMessage, CanTable_Receive, 7);

    // *************************************************************************
    // Active Aero Control Limit Switches - Right Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e06;

    ctSetMessage(&stMessage, CanTable_Receive, 8);

    // *************************************************************************
    // Time Trigger Sync Message (from ECU / PT bus
    canInitMessage(&stMessage);
    stMessage.iId = 0x3200;

    ctSetMessage(&stMessage, CanTable_Receive, 9);

    // *************************************************************************
    // PE1 - RPM, TPS, Fuel Open Time, Ignition Angle
    canInitMessage(&stMessage);
    stMessage.iId = 0x0cfff048;

    ctSetMessage(&stMessage, CanTable_Receive, 10);

    // *************************************************************************
    // PE5 - Frequency 1-4
    canInitMessage(&stMessage);
    stMessage.iId = 0x0cfff448;

    ctSetMessage(&stMessage, CanTable_Receive, 11);

    // *************************************************************************
    // PE6 - Voltage, Air/Coolant Temp, Temp Format
    canInitMessage(&stMessage);
    stMessage.iId = 0x0cfff548;

    ctSetMessage(&stMessage, CanTable_Receive, 12);

    // *************************************************************************
    // Shutdown broadcast from supervisor
    canInitMessage(&stMessage);
    stMessage.iId = 0x200;

    ctSetMessage(&stMessage, CanTable_Receive, 13);

    return;
}

void initRxbuffer(void)
{
	int i,j;
	for(i = 1;i < 4;i++)
	{
		for(j = 0;j < 8;j++)
			aCAN1Buffer[i][j] = 0;
	}
}

void C1_in(uint16_t RBIF, uint16_t RXFUL1, uint16_t RXFUL2, uint16_t RXFUL3, uint32_t id,
			uint16_t *BF)
{
	int i;
	C1INTFbits_RBIF = RBIF;
	C1RXFUL1bits_RXFUL1 = RXFUL1;
	C1RXFUL1bits_RXFUL2 = RXFUL2;
	C1RXFUL1bits_RXFUL3 = RXFUL3;
	id_acm = id;
	
	if(C1RXFUL1bits_RXFUL1 == 1)
	{
		for(i = 0; i < 8;i++)
			aCAN1Buffer[1][i] = BF[i];
	}
	if(C1RXFUL1bits_RXFUL2 == 1)
	{
		for(i = 0; i < 8;i++)
			aCAN1Buffer[2][i] = BF[i];
	}
	if(C1RXFUL1bits_RXFUL3 == 1)
	{
		for(i = 0; i < 8;i++)
			aCAN1Buffer[3][i] = BF[i];
	}
}


void C1_Data_Out(int n)
{
	CAN1_OUT[0] = aCanTableRx[n].iId;
	CAN1_OUT[1] = aCanTableRx[n].bEid;
	CAN1_OUT[2] = aCanTableRx[n].aData[0];
	CAN1_OUT[3] = aCanTableRx[n].aData[1];
	CAN1_OUT[4] = aCanTableRx[n].aData[2];
	CAN1_OUT[5] = aCanTableRx[n].aData[3];
	CAN1_OUT[6] = aCanTableRx[n].aData[4];
	CAN1_OUT[7] = aCanTableRx[n].aData[5];
	CAN1_OUT[8] = aCanTableRx[n].aData[6];
	CAN1_OUT[9] = aCanTableRx[n].aData[7];
	CAN1_OUT[10] = aCanTableRx[n].iDlc;
	CAN1_OUT[11] = aCanTableRx[n].iFilter;
}

uint32_t C1_out(int n)
{
	return CAN1_OUT[n];
}


// This is the CAN event interrupt handler for bus master nodes.  Master nodes
// do not respond to the 0x3200 or 0x3202 sync frame IDs.  The ACM is the bus
// master for the aero bus which is connected to its CAN1 module.

// The ACM transmits 0x3200 to synchronize the aero bus.
void acmIsrC1(void)
{
    #ifdef ENABLE_MODULE_ECAN_1

    uint16_t iCanTableIndex = 0;

    CanMessage stMessage;
	if(C1INTFbits_RBIF)
    // if(C1INTFbits.RBIF)
    {
        canInitMessage(&stMessage);

        // ********************************************************************
        // Process Extended ID messages from ECAN buffer 1 = RX buffer 0

        // if(C1RXFUL1bits.RXFUL1)
		if(C1RXFUL1bits_RXFUL1)
        {
            #ifdef SIMULATION
			// can1RxMessage(1, &stMessage);
			directRxMessage(1, &stMessage);
			#endif
			
			#ifdef REAL
			directRxMessage(1, &stMessage);
			#endif
            // C1RXFUL1bits.RXFUL1 = 0;
			C1RXFUL1bits_RXFUL1 = 0;
            // Store this message in the CAN message table
            iCanTableIndex = ctGetMessageIndex(CanTable_Receive, stMessage.iId);
            ctSetMessage(&stMessage, CanTable_Receive, iCanTableIndex);
        }

        // ********************************************************************
        // Process Extended ID messages from ECAN buffer 2 = RX buffer 1

        // if(C1RXFUL1bits.RXFUL2)
		if(C1RXFUL1bits_RXFUL2)
        {
            #ifdef SIMULATION
			// can1RxMessage(2, &stMessage);
			directRxMessage(2, &stMessage);
			#endif
			
			#ifdef REAL
			directRxMessage(2, &stMessage);
			#endif
            // C1RXFUL1bits.RXFUL2 = 0;
			C1RXFUL1bits_RXFUL2 = 0;
            // Store this message in the CAN message table
            iCanTableIndex = ctGetMessageIndex(CanTable_Receive, stMessage.iId);
            ctSetMessage(&stMessage, CanTable_Receive, iCanTableIndex);
        }

        // ********************************************************************
        // Process Extended ID messages from ECAN buffer 3 = RX buffer 2

        // if(C1RXFUL1bits.RXFUL3)
		if(C1RXFUL1bits_RXFUL3)
        {
            #ifdef SIMULATION
			// can1RxMessage(3, &stMessage);
			directRxMessage(3, &stMessage);
			#endif
			
			#ifdef REAL
			directRxMessage(3, &stMessage);
			#endif
            // C1RXFUL1bits.RXFUL3 = 0;
			C1RXFUL1bits_RXFUL3 = 0;
            // Store this message in the CAN message table
            iCanTableIndex = ctGetMessageIndex(CanTable_Receive, stMessage.iId);
            ctSetMessage(&stMessage, CanTable_Receive, iCanTableIndex);
        }
		C1_Data_Out(iCanTableIndex);
        // Clear ECAN module TX interrupt flag
        // C1INTFbits.RBIF = 0;
		C1INTFbits_RBIF = 0;
    }
    // else if(C1INTFbits.TBIF)
    // {
        // Clear ECAN module TX interrupt flag
        // C1INTFbits.TBIF = 0;
    // }

    #endif

    return;
}

