// Timer 1 - TTCAN1 Master Timer, this version is
// for stateflow, no mex function involved.
// #include "stdint.h"
#include "acm.h"
#include "acmIsrT1.h"

uint8_t iACMTTCan1Cycle = 0;
uint8_t iACMTTCan1Window = 0;

// Dummy window array for use with e15aerTtCanInitSchedule().
uint32_t aE15AerDummy[3];

// *****************************************************************************
// Aero Master

uint32_t aAcmDatAer[7];     // Aero master data
uint32_t aAcmSrvAer[4];     // Servo positions

uint32_t aAcmSycAer[3];     // TTCAN sync frame

// *****************************************************************************
// Aero Wings

uint32_t aAw0LswAer[3];     // Aero wing 0 limit switches
uint32_t aAw1LswAer[3];     // Aero wing 1 limit switches
uint32_t aAw2LswAer[3];     // Aero wing 2 limit switches
uint32_t aAw3LswAer[3];     // Aero wing 3 limit switches

// *****************************************************************************
// Heartbeats

uint32_t aAcmHbtAer[3];     // Aero master

uint32_t aAw0HbtAer[3];     // Aero wing 0 (left front)
uint32_t aAw1HbtAer[3];     // Aero wing 1 (right front)
uint32_t aAw2HbtAer[3];     // Aero wing 2 (left rear)
uint32_t aAw3HbtAer[3];     // Aero wing 3 (right rear)

CanMessage aCanTableRx[CAN_TABLE_RX_LENGTH];
CanMessage aCanTableTx[CAN_TABLE_TX_LENGTH];
CanMessage stDeadMessage;										  


/*
// If a user application doesn't specific any timer start/stop functions, this
// function is used as a default for both.
void can1TimerDummy(void)
{
    return;
}


// Use these function pointers to control a timer that call canTxCancel() in an
// ISR.  This gives the CAN HAL the ability to cancel transmission if it takes
// too long.
void (*can1TxTimerStart)(void) = can1TimerDummy;
void (*can1TxTimerStop)(void) = can1TimerDummy;

void (*can2TxTimerStart)(void) = can2TimerDummy;
void (*can2TxTimerStop)(void) = can2TimerDummy;
*/
										  



// Initialize each window's CAN IDs.  Note that these are just arrays, they have
// no accompanying array index.  Since other TTCAN functions need to read these
// arrays, the current array index needs to be persistent between function
// calls.

// I didn't want to mess with trying to create a struct that holds both an
// array and index, because I don't want to use dynamic memory allocation to
// create arrays of arbitrary length.  The other option was to statically
// allocate the arrays within each struct object.  Since I can only specify one
// array length per struct definition, that would be a huge waste of RAM.

// So, I had to get creative.  These arrays are formatted as follows:
// aArray[0] - element 0 is the current array index, initially zero.
// aArray[1 to n-1] - the CAN IDs.
// aArray[n] - a zero value that terminates the array.

void e15aerTtCanInitIds(void)
{
    // Dummy window array for use with e15aerTtCanInitSchedule().
    aE15AerDummy[0] = 1;
    aE15AerDummy[1] = 0x00;
    aE15AerDummy[2] = 0;

    // *************************************************************************
    // Aero Master

    // Aero master data
    aAcmDatAer[0] = 1;
    aAcmDatAer[1] = 0x1a00;
    aAcmDatAer[2] = 0x1c00;
    aAcmDatAer[3] = 0x1c02;
    aAcmDatAer[4] = 0x1c04;
    aAcmDatAer[5] = 0x1c06;
    aAcmDatAer[6] = 0;

    aAcmSrvAer[0] = 1;
    aAcmSrvAer[1] = 0x1c08;
    aAcmSrvAer[2] = 0x1c0a;
    aAcmSrvAer[3] = 0;

    // TTCAN sync frame
    aAcmSycAer[0] = 1;
    aAcmSycAer[1] = 0x3200;
    aAcmSycAer[2] = 0;

    // *************************************************************************
    // Aero Wings

    // Aero wing 0 limit switches
    aAw0LswAer[0] = 1;
    aAw0LswAer[1] = 0x1e00;
    aAw0LswAer[2] = 0;

    // Aero wing 1 limit switches
    aAw1LswAer[0] = 1;
    aAw1LswAer[1] = 0x1e02;
    aAw1LswAer[2] = 0;

    // Aero wing 2 limit switches
    aAw2LswAer[0] = 1;
    aAw2LswAer[1] = 0x1e04;
    aAw2LswAer[2] = 0;

    // Aero wing 3 limit switches
    aAw3LswAer[0] = 1;
    aAw3LswAer[1] = 0x1e06;
    aAw3LswAer[2] = 0;

    // *************************************************************************
    // Heartbeats

    // Aero master (to aero bus)
    aAcmHbtAer[0] = 1;
    aAcmHbtAer[1] = 0x460;
    aAcmHbtAer[0] = 0;

    // Aero wing 0 (left front)
    aAw0HbtAer[0] = 1;
    aAw0HbtAer[1] = 0x462;
    aAw0HbtAer[2] = 0;

    // Aero wing 1 (right front)
    aAw1HbtAer[0] = 1;
    aAw1HbtAer[1] = 0x464;
    aAw1HbtAer[2] = 0;

    // Aero wing 2 (left rear)
    aAw2HbtAer[0] = 1;
    aAw2HbtAer[1] = 0x466;
    aAw2HbtAer[2] = 0;

    // Aero wing 3 (right rear)
    aAw3HbtAer[0] = 1;
    aAw3HbtAer[1] = 0x468;
    aAw3HbtAer[2] = 0;

    return;
}


// Initialize the schedule's window objects.
void e15aerTtCanInitSchedule(void)
{
    uint8_t iCycleIndex = 0;
    uint8_t iWindowIndex = 0;

    // Populate all window objects with blank data.
    for(iCycleIndex = 0; iCycleIndex < E15AER_TTCAN_SCHEDULE_CYCLES; iCycleIndex++)
    {
        for(iWindowIndex = 0; iWindowIndex < E15AER_TTCAN_SCHEDULE_WINDOWS; iWindowIndex++)
        {
            aE15AerSchedule[iCycleIndex][iWindowIndex].aMessageIds = aE15AerDummy;
            aE15AerSchedule[iCycleIndex][iWindowIndex].iNodeAddress = 0x00;
        }
    }

    // ************************************************************************
    // Cycle 0

    // Window 0 - TTCAN Sync - Supervisor
    aE15AerSchedule[0][0].aMessageIds = aAcmSycAer;		//3200
    aE15AerSchedule[0][0].iNodeAddress = 0x460;

    // Window 1 - Data Frame - AAC Master
    aE15AerSchedule[0][1].aMessageIds = aAcmDatAer;		// 1a00, 1c00, 1c02, 1c04, 1c06
    aE15AerSchedule[0][1].iNodeAddress = 0x460;

    // Window 2 - Front Servo Position - AAC Master
    aE15AerSchedule[0][2].aMessageIds = aAcmSrvAer;		// 1c08, 1c0a
    aE15AerSchedule[0][2].iNodeAddress = 0x460;

    // Window 3 - Rear Servo Position - AAC Master
    aE15AerSchedule[0][3].aMessageIds = aAcmSrvAer;		// 1c08, 1c0a
    aE15AerSchedule[0][3].iNodeAddress = 0x460;

    // Window 4 - Heartbeat - AAC Wing LF
    aE15AerSchedule[0][4].aMessageIds = aAw0HbtAer;
    aE15AerSchedule[0][4].iNodeAddress = 0x462;

    // Window 5 - Heartbeat - AAC Wing RF
    aE15AerSchedule[0][5].aMessageIds = aAw1HbtAer;
    aE15AerSchedule[0][5].iNodeAddress = 0x464;

    // Window 6 - Limit Switches - AAC Wing LF
    aE15AerSchedule[0][6].aMessageIds = aAw0LswAer;
    aE15AerSchedule[0][6].iNodeAddress = 0x462;

    // Window 7 - Limit Switches - AAC Wing RF
    aE15AerSchedule[0][7].aMessageIds = aAw1LswAer;
    aE15AerSchedule[0][7].iNodeAddress = 0x464;

    // Window 8 - Limit Switches - AAC Wing LR
    aE15AerSchedule[0][8].aMessageIds = aAw2LswAer;
    aE15AerSchedule[0][8].iNodeAddress = 0x466;

    // Window 9 - Limit Switches - AAC Wing RR
    aE15AerSchedule[0][9].aMessageIds = aAw3LswAer;		// 1e06
    aE15AerSchedule[0][9].iNodeAddress = 0x468;

    // ************************************************************************
    // Cycle 1

    // Window 0 - TTCAN Sync - Supervisor
    aE15AerSchedule[1][0].aMessageIds = aAcmSycAer;
    aE15AerSchedule[1][0].iNodeAddress = 0x460;

    // Window 1 - Data Frame - AAC Master
    aE15AerSchedule[1][1].aMessageIds = aAcmDatAer;
    aE15AerSchedule[1][1].iNodeAddress = 0x460;

    // Window 2 - Front Servo Position - AAC Master
    aE15AerSchedule[1][2].aMessageIds = aAcmSrvAer;
    aE15AerSchedule[1][2].iNodeAddress = 0x460;

    // Window 3 - Rear Servo Position - AAC Master
    aE15AerSchedule[1][3].aMessageIds = aAcmSrvAer;
    aE15AerSchedule[1][3].iNodeAddress = 0x460;

    // Window 4 - Heartbeat - AAC Wing LR
    aE15AerSchedule[1][4].aMessageIds = aAw2HbtAer;
    aE15AerSchedule[1][4].iNodeAddress = 0x466;

    // Window 5 - Heartbeat - AAC Wing RR
    aE15AerSchedule[1][5].aMessageIds = aAw3HbtAer;
    aE15AerSchedule[1][5].iNodeAddress = 0x468;

    // Window 6 - Limit Switches - AAC Wing LF
    aE15AerSchedule[1][6].aMessageIds = aAw0LswAer;
    aE15AerSchedule[1][6].iNodeAddress = 0x462;

    // Window 7 - Limit Switches - AAC Wing RF
    aE15AerSchedule[1][7].aMessageIds = aAw1LswAer;
    aE15AerSchedule[1][7].iNodeAddress = 0x464;

    // Window 8 - Limit Switches - AAC Wing LR
    aE15AerSchedule[1][8].aMessageIds = aAw2LswAer;
    aE15AerSchedule[1][8].iNodeAddress = 0x466;

    // Window 9 - Limit Switches - AAC Wing RR
    aE15AerSchedule[1][9].aMessageIds = aAw3LswAer;
    aE15AerSchedule[1][9].iNodeAddress = 0x468;

    // ************************************************************************
    // Cycle 2

    // Window 0 - TTCAN Sync - Supervisor
    aE15AerSchedule[2][0].aMessageIds = aAcmSycAer;
    aE15AerSchedule[2][0].iNodeAddress = 0x460;

    // Window 1 - Data Frame - AAC Master
    aE15AerSchedule[2][1].aMessageIds = aAcmDatAer;
    aE15AerSchedule[2][1].iNodeAddress = 0x460;

    // Window 2 - Front Servo Position - AAC Master
    aE15AerSchedule[2][2].aMessageIds = aAcmSrvAer;
    aE15AerSchedule[2][2].iNodeAddress = 0x460;

    // Window 3 - Rear Servo Position - AAC Master
    aE15AerSchedule[2][3].aMessageIds = aAcmSrvAer;
    aE15AerSchedule[2][3].iNodeAddress = 0x460;

    // Window 4 - Heartbeat - AAC Master
    aE15AerSchedule[2][4].aMessageIds = aAcmHbtAer;		// 460
    aE15AerSchedule[2][4].iNodeAddress = 0x460;

    // Window 5 - Reserved

    // Window 6 - Limit Switches - AAC Wing LF
    aE15AerSchedule[2][6].aMessageIds = aAw0LswAer;
    aE15AerSchedule[2][6].iNodeAddress = 0x462;

    // Window 7 - Limit Switches - AAC Wing RF
    aE15AerSchedule[2][7].aMessageIds = aAw1LswAer;
    aE15AerSchedule[2][7].iNodeAddress = 0x464;

    // Window 8 - Limit Switches - AAC Wing LR
    aE15AerSchedule[2][8].aMessageIds = aAw2LswAer;
    aE15AerSchedule[2][8].iNodeAddress = 0x466;

    // Window 9 - Limit Switches - AAC Wing RR
    aE15AerSchedule[2][9].aMessageIds = aAw3LswAer;
    aE15AerSchedule[2][9].iNodeAddress = 0x468;

    return;
}


uint16_t acmCanGetAddress(void)
{
    return (uint16_t)ACM_CAN_ADDRESS;
}

// Retrieve the current TTCAN1 cycle index for CAN1.
uint8_t * acmCan1GetCycle(void)
{
    return &iACMTTCan1Cycle;
}

// Retrieve the current TTCAN window index for CAN1.
uint8_t * acmCan1GetWindow(void)
{
    return &iACMTTCan1Window;
}
// Initial Tx table
void acmCanInitTableTx(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // Heartbeat - ACM
    canInitMessage(&stMessage);
    stMessage.iId = ACM_CAN_ADDRESS;
	stMessage.aData[0] = 3;

    ctSetMessage(&stMessage, CanTable_Transmit, 0);

    // *************************************************************************
    // Active Aero Control Sensors - Switches, Accelerometer XYZ
    canInitMessage(&stMessage);
    stMessage.iId = 0x1a00;

    ctSetMessage(&stMessage, CanTable_Transmit, 1);

    // *************************************************************************
    // Active Aero Control Servo Range - Left Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c00;

    ctSetMessage(&stMessage, CanTable_Transmit, 2);

    // *************************************************************************
    // Active Aero Control Servo Range - Right Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c02;

    ctSetMessage(&stMessage, CanTable_Transmit, 3);

    // *************************************************************************
    // Active Aero Control Servo Range - Left Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c04;

    ctSetMessage(&stMessage, CanTable_Transmit, 4);

    // *************************************************************************
    // Active Aero Control Servo Range - Right Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c06;

    ctSetMessage(&stMessage, CanTable_Transmit, 5);

    // *************************************************************************
    // Active Aero Control Servo Position - Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c08;

    ctSetMessage(&stMessage, CanTable_Transmit, 6);

    // *************************************************************************
    // Active Aero Control Servo Position - Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c0a;

    ctSetMessage(&stMessage, CanTable_Transmit, 7);

    // *************************************************************************
    // Active Aero System State
    canInitMessage(&stMessage);
    stMessage.iId = 0x1c10;

    ctSetMessage(&stMessage, CanTable_Transmit, 8);

    // *************************************************************************
    // Network Management - Time Trigger Synchronization Message
    canInitMessage(&stMessage);
    stMessage.iDlc = 1;
    stMessage.iId = 0x3200;
    
    ctSetMessage(&stMessage, CanTable_Transmit, 9);
	
	// *************************************************************************
    // Shutdown (for aero bus)
    canInitMessage(&stMessage);
    stMessage.iId = 0x200;

    ctSetMessage(&stMessage, CanTable_Transmit, 10);

    return;
}

// Unlike the MCP2515, the PIC24H ECAN module expects the user application to
// provide the arbitration field (SID or EID, and the RTR bit), control field
// (IDE, RB0, and Data Length Code) and data field.  The ECAN module calculates
// the CRC and then fills in ACK and EOF by itself.  See pages 3 and 17 of the
// DS01249A reference document.

// The CAN DMA buffers are 8 items long, each item is 16 bytes.
void can1TxMessage(CanMessage * stCanMessage)
{
    static uint16_t aCAN1BufferWord[8];
    uint8_t iBufferIndex = 0;   // Index for aCAN1BufferWord
    uint8_t iDataIndex = 0;     // Index for aData within stCanMessage    
	#ifdef ENABLE_MODULE_ECAN_1

    // if(stCanMessage == NULL)
        // return;

    // static uint16_t aCAN1BufferWord[8];		//move this piece of code out of marco
    // uint8_t iBufferIndex = 0;   // Index for aCAN1BufferWord
    // uint8_t iDataIndex = 0;     // Index for aData within stCanMessage

    // Validate the Data Length Code.
    if(stCanMessage->iDlc > 8)
        stCanMessage->iDlc = 0;

    // Initialize the buffer word array.
    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
        aCAN1BufferWord[iBufferIndex] = 0;

    // Refer to pages 9 and 10 of the MCP2515 datasheet for a detailed field
    // and bit description.

    // ********************************************************************
    // ECAN Message Buffer Word 0 - PIC24H FRM, page 21-30

    // Add in the 11 most-significant bits of the message ID.
    // For a standard ID message, this would be all 11 bits in the ID.
    // For an extended ID message, this would be ID bits 29 through 18.
    if(stCanMessage->bEid)
        aCAN1BufferWord[0] = (stCanMessage->iId >> 18);
    else
        aCAN1BufferWord[0] = stCanMessage->iId;

    // Make room for SRR and IDE bits
    aCAN1BufferWord[0] = aCAN1BufferWord[0] << 2;

    if(stCanMessage->bEid)
    {
        // For extended CAN IDs, the Substitute-Remote Request (SRR) bit and the
        // IDE bits must be 1.  IDE = 1 means the message has an extended ID.
        aCAN1BufferWord[0] |= 0x02;   // Set SRR bit
        aCAN1BufferWord[0] |= 0x01;   // Set IDE bit
    }
    else
    {
        // For standard CAN IDs, the SRR bit doesn't matter and the IDE bit
        // must be cleared.
        aCAN1BufferWord[0] &= ~(0x02);   // Clear SRR bit
        aCAN1BufferWord[0] &= ~(0x01);   // Clear IDE bit
    }

    // ********************************************************************
    // ECAN Message Buffer Word 1 - PIC24H FRM, page 21-30

    // Next, add in bits 17 through 6 of the extended ID.
    if(stCanMessage->bEid)
        aCAN1BufferWord[1] = (stCanMessage->iId >> 6) & 0x0fff;
    
    // ********************************************************************
    // ECAN Message Buffer Word 2 - PIC24H FRM, page 21-31

    if(stCanMessage->bEid)
    {
        // Next, add in bits 5 through 0 of the extended ID.
        aCAN1BufferWord[2] = stCanMessage->iId;

        // Shift the data over to make room for the other 11 buffer bits.
        aCAN1BufferWord[2] = (aCAN1BufferWord[2] << 10); 
    }

    // Add the Remote Transmission Request RTR bit.  Since this is not
    // an RTR frame, the bit will be 0.
    aCAN1BufferWord[2] &= ~(0x200);

    // Set Reserve Bit 1 (RB1) to 0.
    aCAN1BufferWord[2] &= ~(0x100);

    // Set Reserve Bit 0 (RB0) to 0.
    aCAN1BufferWord[2] &= ~(0x10);

    // Add the Data Length Code (DLC) bits.
    aCAN1BufferWord[2] |= stCanMessage->iDlc;

    // ********************************************************************
    // ECAN Message Buffer Words 3 to 6 - PIC24H FRM, page 21-31 to 32

    // Add CAN message data to the DMA buffers.
    for(iDataIndex = 0; iDataIndex < stCanMessage->iDlc; iDataIndex++)
        aCAN1BufferWord[3 + (iDataIndex / 2)] |= (stCanMessage->aData[iDataIndex] << ((iDataIndex % 2) * 8));

    // ********************************************************************
    // ECAN Message Buffer Word 7 - PIC24H FRM, page 21-33
    // Buffer Word 7 is not used when sending messages.

    // ********************************************************************
    // Copy the CAN buffer words into the DMA buffer
    // Remember, DMA buffer 0 has been setup as an ECAN TX buffer.

    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
        aCAN1Buffer[0][iBufferIndex] = aCAN1BufferWord[iBufferIndex];

    // ********************************************************************
    // Set request to transmit

    // Once a transmission buffer has been loaded with message data, the TXREQ
    // flag in the buffer's control register must be set.  Setting this bit
    // does not guarantee that the CAN message has been sent.  The message will
    // be sent when the CAN bus becomes available.  See page 21-28 of the
    // PIC24H FRM.  To access this register, the WIN bit must be cleared.

// below is H/W operation
/*
    C1CTRL1bits.WIN = 0;			//H/W related operation

    // Transmit from DMA buffer 0 / ECAN TX buffer 0
    C1TR01CONbits.TXREQ0 = 1;	//H/W related operation
    (*can1TxTimerStart)();

    while(C1TR01CONbits.TXREQ0)
    {
        // Check for a transmission error.
        if(C1TR01CONbits.TXERR0 || C1TR01CONbits.TXLARB0)
        {
            C1TR01CONbits.TXREQ0 = 0;
            break;
        }
    }

    (*can1TxTimerStop)();
*/
    #endif

    return;
}

// This function is only for simulation purpose.
void directTxMessage(CanMessage * stCanMessage)
{
    static uint16_t aCAN1BufferWord[8];
    uint8_t iBufferIndex = 0;   // Index for aCAN1BufferWord
    uint8_t iDataIndex = 0;     // Index for aData within stCanMessage    
	#ifdef ENABLE_MODULE_ECAN_1

    // if(stCanMessage == NULL)
        // return;

    // Validate the Data Length Code.
    if(stCanMessage->iDlc > 8)
        stCanMessage->iDlc = 0;

    // Initialize the buffer word array.
    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
        aCAN1BufferWord[iBufferIndex] = 0;
	
	for(iDataIndex = 0; iDataIndex < 8; iDataIndex++)
		aCAN1BufferWord[iDataIndex] = stCanMessage->aData[iDataIndex];
	
    // ********************************************************************
    // Copy the CAN buffer words into the DMA buffer
    // Remember, DMA buffer 0 has been setup as an ECAN TX buffer.

    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
		aCAN1Buffer[0][iBufferIndex] = aCAN1BufferWord[iBufferIndex] & 0xff;
	#endif
	return;
}


// Timer 1 - TTCAN schedule timer.  This version of the function is for
// supervisor nodes. Timer 1 - TTCAN1 Master Timer
void acmIsrT1(void)
{
    uint32_t iCanId = 0x000;

    uint16_t iCanAddress;
    uint8_t * iTTCanCycle;
    uint8_t * iTTCanWindow;

    CanMessage * stMessage;

    // Abort previous window transmission.
    can1AbortTx(0);

    iCanAddress = acmCanGetAddress();
    iTTCanCycle = acmCan1GetCycle();
    iTTCanWindow = acmCan1GetWindow();

    // Retrieve the message ID from the TTCAN schedule
    iCanId = e15aerTtCanGetId(iCanAddress, *iTTCanCycle, *iTTCanWindow);
    stMessage = ctGetMessageById(CanTable_Transmit, iCanId);
    
    // Update TTCAN sync frame (if applicable)
    if(iCanId == 0x3200){
        stMessage->aData[0] = *iTTCanCycle;
		stMessage->aData[1] = 3;
    }
    // *************************************************************************
    // Transmission time!
    if(iCanId != 0x00)
    {
        // acmCan1RxLedOn();

        // can1TxMessage(stMessage);
		directTxMessage(stMessage);
		T1_Data_Out(iCanId, *iTTCanCycle, *iTTCanWindow);
    }    
    // else
        // acmCan1RxLedOff();
    
    // *************************************************************************
    // TTCAN update time!
    
    // Increment the window
    (*iTTCanWindow)++;

    // If this is the last window in the cycle, increment the cycle index.
    if((*iTTCanWindow) >= E15AER_TTCAN_SCHEDULE_WINDOWS)
    {
        // Loop the cycle index continuously.
        (*iTTCanCycle)++;

        if((*iTTCanCycle) >= E15AER_TTCAN_SCHEDULE_CYCLES)
            (*iTTCanCycle) = 0;

        // Store the cycle index in the TTCAN sync message.
        stMessage = ctGetMessageById(CanTable_Transmit, 0x3200);
        stMessage->aData[0] = *iTTCanCycle;
		stMessage->aData[1] = 3;
        (*iTTCanWindow) = 0;

        // acmCan1RxLedOff();
    }

    return;
}


void T1_Data_Out(uint32_t canid, uint8_t cycle, uint8_t window)
{
	int i;
	for(i=0;i<11;i++)
		TIMER1_OUT[i] = 0;
	for(i=0;i<8;i++)
		TIMER1_OUT[i] = aCAN1Buffer[0][i]&0x00ffff;
	TIMER1_OUT[8] = canid;
	TIMER1_OUT[9] = cycle;
	TIMER1_OUT[10] = window;
}

uint16_t T1_out(uint8_t n)
{
	return TIMER1_OUT[n];
}

/*
// this function is only for testing purpose
void SetTableTx()
{			
	uint8_t i,j;
	CanMessage *stMessage;
	// canInitMessage(&stMessage);
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1a00);		// get the current message. because we only modify the aData field, 
	stMessage->aData[0] = 0x1a;										// so we must keep other fields the same.
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c00);
	stMessage->aData[0] = 0x1c;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c02);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x02;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c04);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x04;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c06);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x06;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c08);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x08;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c0a);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x0a;
	
	stMessage = ctGetMessageById(CanTable_Transmit, 0x1c10);
	stMessage->aData[0] = 0x1c;stMessage->aData[1] = 0x10;
	// stMessage = ctGetMessageById(CanTable_Transmit, 0x3200);
	// stMessage->aData[0] = 0x32;
	
	// here, the stMessage is a pointer points to the corresponding slot of aCanTableTx,
	// so no need to use ctSetMessage() to set value

}*/











