#include "acm.h"
#include "acmIsrT3.h"

uint16_t TIMER3_OUT;
// Look for a duplicate heartbeat.
void acmCanInputDuplicateHb(void)
{
    AcmSensorData * stAcmSensor = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Receive, ACM_CAN_ADDRESS);

    if(stMessage->iId == ACM_CAN_ADDRESS)
    {
        if(stMessage->bPending == 1 && stMessage->bWriting == 0)
        {
            stAcmSensor->bDuplicateHb = 1;
            stMessage->bPending = 0;
        }
    }

    return;
}

// Reset the heartbeat timers.
void acmCanInputHeartbeat(void)			// since we only have 0x468 here so ignore other nodes.
{
    CanMessage * stMessage = ctGetMessageById(CanTable_Receive, 0x0);

    // *************************************************************************
    // Left front wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x462);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        heartbeat_mwSet(ACM_HEARTBEAT_TIMER_ACW_LF, ACM_HEARTBEAT_TIMEOUT_MULTIPLIER);
        stMessage->bPending = 0;
    }

    // *************************************************************************
    // Right front wing
/*    stMessage = ctGetMessageById(CanTable_Receive, 0x464);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        heartbeat_mwSet(ACM_HEARTBEAT_TIMER_ACW_RF, ACM_HEARTBEAT_TIMEOUT_MULTIPLIER);
        stMessage->bPending = 0;
    }

    // *************************************************************************
    // Left rear wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x466);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        heartbeat_mwSet(ACM_HEARTBEAT_TIMER_ACW_LR, ACM_HEARTBEAT_TIMEOUT_MULTIPLIER);
        stMessage->bPending = 0;
    }
*/
    // *************************************************************************
    // Right rear wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x468);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        heartbeat_mwSet(ACM_HEARTBEAT_TIMER_ACW_RR, ACM_HEARTBEAT_TIMEOUT_MULTIPLIER);
        stMessage->bPending = 0;
    }

    return;
}

void acmCanInputShutdown(void)
{
    AcmSensorData * stAcmSensor = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Receive, 0x200);
    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        if(stMessage->aData[0] != 0xa5)
            return;

        stAcmSensor->bShutdown = 1;

        stMessage->bPending = 0;
    }

    return;
}

void acmCanInputSwLimit(void)
{
    AcmSensorData * stAcmSensor = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Receive, 0x0cfff448);

    uint8_t iSwLimit = 0;

    // Left Front Wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x1e00);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        stMessage->bReading = 1;

        iSwLimit |= (stMessage->aData[0] & 0x01) << (ACM_WING_LEFT_FRONT * 2);
        iSwLimit |= (stMessage->aData[0] & 0x02) << (ACM_WING_LEFT_FRONT * 2);

        stMessage->bReading = 0;
        stMessage->bPending = 0;
    }

    // Right Front Wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x1e02);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        stMessage->bReading = 1;

        iSwLimit |= (stMessage->aData[0] & 0x01) << (ACM_WING_RIGHT_FRONT * 2);
        iSwLimit |= (stMessage->aData[0] & 0x02) << (ACM_WING_RIGHT_FRONT * 2);

        stMessage->bReading = 0;
        stMessage->bPending = 0;
    }

    // Left Rear Wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x1e04);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        stMessage->bReading = 1;

        iSwLimit |= (stMessage->aData[0] & 0x01) << (ACM_WING_LEFT_REAR * 2);
        iSwLimit |= (stMessage->aData[0] & 0x02) << (ACM_WING_LEFT_REAR * 2);

        stMessage->bReading = 0;
        stMessage->bPending = 0;
    }

    // Right Rear Wing
    stMessage = ctGetMessageById(CanTable_Receive, 0x1e06);

    if(stMessage->bPending == 1 && stMessage->bWriting == 0)
    {
        stMessage->bReading = 1;
        
        iSwLimit |= (stMessage->aData[0] & 0x01) << (ACM_WING_RIGHT_REAR * 2);
        iSwLimit |= (stMessage->aData[0] & 0x02) << (ACM_WING_RIGHT_REAR * 2);

        stMessage->bReading = 0;
        stMessage->bPending = 0;
    }

    stAcmSensor->iSwLimit = iSwLimit;

    return;
}

void acmCanOutputServoLimits(void)
{
    ServoData * aAcmServo = acmServoRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x0);

    // *************************************************************************
    // Left Front Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c08);
    stMessage->bWriting = 1;

    // Left Front Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_LFL].iPosCurrent >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_LFL].iPosCurrent;

    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LFL].bEnabled & 0x01) << 4;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LFL].bInvert & 0x01) << 5;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LFL].bMoving & 0x01) << 6;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LFL].bPowered & 0x01) << 7;

    // Left Front Upper
    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_LFU].iPosCurrent >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_LFU].iPosCurrent;

    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LFU].bEnabled & 0x01) << 4;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LFU].bInvert & 0x01) << 5;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LFU].bMoving & 0x01) << 6;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LFU].bPowered & 0x01) << 7;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Right Front Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c08);
    stMessage->bWriting = 1;

    // Right Front Lower
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_RFL].iPosCurrent >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_RFL].iPosCurrent;

    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RFL].bEnabled & 0x01) << 4;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RFL].bInvert & 0x01) << 5;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RFL].bMoving & 0x01) << 6;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RFL].bPowered & 0x01) << 7;

    // Right Front Upper
    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_RFU].iPosCurrent >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_RFU].iPosCurrent;

    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RFU].bEnabled & 0x01) << 4;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RFU].bInvert & 0x01) << 5;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RFU].bMoving & 0x01) << 6;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RFU].bPowered & 0x01) << 7;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Left Rear Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c0a);
    stMessage->bWriting = 1;

    // Left Rear Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_LRL].iPosCurrent >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_LRL].iPosCurrent;

    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LRL].bEnabled & 0x01) << 4;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LRL].bInvert & 0x01) << 5;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LRL].bMoving & 0x01) << 6;
    stMessage->aData[0] |= (aAcmServo[ACM_SERVO_LRL].bPowered & 0x01) << 7;

    // Left Rear Upper
    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_LRU].iPosCurrent >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_LRU].iPosCurrent;

    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LRU].bEnabled & 0x01) << 4;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LRU].bInvert & 0x01) << 5;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LRU].bMoving & 0x01) << 6;
    stMessage->aData[2] |= (aAcmServo[ACM_SERVO_LRU].bPowered & 0x01) << 7;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Right Rear Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c0a);
    stMessage->bWriting = 1;

    // Right Rear Lower
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_RRL].iPosCurrent >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_RRL].iPosCurrent;

    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RRL].bEnabled & 0x01) << 4;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RRL].bInvert & 0x01) << 5;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RRL].bMoving & 0x01) << 6;
    stMessage->aData[4] |= (aAcmServo[ACM_SERVO_RRL].bPowered & 0x01) << 7;

    // Right Rear Upper
    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_RRU].iPosCurrent >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_RRU].iPosCurrent;

    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RRU].bEnabled & 0x01) << 4;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RRU].bInvert & 0x01) << 5;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RRU].bMoving & 0x01) << 6;
    stMessage->aData[6] |= (aAcmServo[ACM_SERVO_RRU].bPowered & 0x01) << 7;

    stMessage->bWriting = 0;

    return;
}

void acmCanOutputServoPosition(void)
{
    ServoData * aAcmServo = acmServoRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x0);

    // *************************************************************************
    // Left Front Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c00);
    stMessage->bWriting = 1;

    // Left Front Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_LFL].iPosMax >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_LFL].iPosMax;

    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_LFL].iPosMin >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_LFL].iPosMin;

    // Left Front Upper
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_LFU].iPosMax >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_LFU].iPosMax;

    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_LFU].iPosMin >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_LFU].iPosMin;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Right Front Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c02);
    stMessage->bWriting = 1;

    // Right Front Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_RFL].iPosMax >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_RFL].iPosMax;

    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_RFL].iPosMin >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_RFL].iPosMin;

    // Right Front Upper
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_RFU].iPosMax >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_RFU].iPosMax;

    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_RFU].iPosMin >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_RFU].iPosMin;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Left Rear Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c04);
    stMessage->bWriting = 1;

    // Left Rear Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_LRL].iPosMax >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_LRL].iPosMax;

    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_LRL].iPosMin >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_LRL].iPosMin;

    // Left Rear Upper
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_LRU].iPosMax >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_LRU].iPosMax;

    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_LRU].iPosMin >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_LRU].iPosMin;

    stMessage->bWriting = 0;

    // *************************************************************************
    // Right Rear Wing
    stMessage = ctGetMessageById(CanTable_Transmit, 0x1c06);
    stMessage->bWriting = 1;

    // Right Rear Lower
    stMessage->aData[0] = (uint8_t)(aAcmServo[ACM_SERVO_RRL].iPosMax >> 8);
    stMessage->aData[1] = (uint8_t)aAcmServo[ACM_SERVO_RRL].iPosMax;

    stMessage->aData[2] = (uint8_t)(aAcmServo[ACM_SERVO_RRL].iPosMin >> 8);
    stMessage->aData[3] = (uint8_t)aAcmServo[ACM_SERVO_RRL].iPosMin;

    // Right Rear Upper
    stMessage->aData[4] = (uint8_t)(aAcmServo[ACM_SERVO_RRU].iPosMax >> 8);
    stMessage->aData[5] = (uint8_t)aAcmServo[ACM_SERVO_RRU].iPosMax;

    stMessage->aData[6] = (uint8_t)(aAcmServo[ACM_SERVO_RRU].iPosMin >> 8);
    stMessage->aData[7] = (uint8_t)aAcmServo[ACM_SERVO_RRU].iPosMin;

    stMessage->bWriting = 0;
    
    return;
}

void acmCanOutputShutdown(void)
{
    AcmSensorData * stAcmSensor = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x200);

    if(stAcmSensor->bShutdown == 1)
    {
        stMessage->bWriting = 1;
        
        stMessage->aData[0] = 0xa5;
        stMessage->aData[1] = 0xa5;
        
        stMessage->bWriting = 0;
    }

    return;
}

// This function writes the system state and the position of the dash switches
// to the CAN table.
void acmCanOutputState(void)
{
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x1c10);
    ACM_State_e * eState = acmStateRet();
    uint8_t * aCalibrateState = acmCalibrateRet();
    
    stMessage->bWriting = 1;

    // Aero run state code
    stMessage->aData[0] = *eState;

    // Aero calibrate state codes
    stMessage->aData[1] = aCalibrateState[ACM_SERVO_LFU] << 4;
    stMessage->aData[1] |= aCalibrateState[ACM_SERVO_LFL];

    stMessage->aData[2] = aCalibrateState[ACM_SERVO_RFU] << 4;
    stMessage->aData[2] |= aCalibrateState[ACM_SERVO_RFL];

    stMessage->aData[3] = aCalibrateState[ACM_SERVO_LRU] << 4;
    stMessage->aData[3] |= aCalibrateState[ACM_SERVO_LRL];

    stMessage->aData[4] = aCalibrateState[ACM_SERVO_RRU] << 4;
    stMessage->aData[4] |= aCalibrateState[ACM_SERVO_RRL];

    stMessage->bWriting = 0;

    return;
}

// This function writes dashboard switch positions to the CAN table.
void acmCanOutputSwExt(void)
{
    AcmSensorData * stSensorData = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x1a00);

    // Pack the sensor data into the CAN message.
    stMessage->aData[7] = stSensorData->iSwExt;

    return;
}

// This function writes accelerometer XYZ values to the CAN table.
void acmCanOutputXyz(void)
{
    AcmSensorData * stSensorData = acmSensorRet();
    CanMessage * stMessage = ctGetMessageById(CanTable_Transmit, 0x1a00);
    
    stMessage->bWriting = 1;

    // Pack the sensor data into the CAN message.
    stMessage->aData[0] = (uint8_t)(stSensorData->stXyzData.iXAccel >> 8);
    stMessage->aData[1] = (uint8_t)stSensorData->stXyzData.iXAccel;

    stMessage->aData[2] = (uint8_t)(stSensorData->stXyzData.iYAccel >> 8);
    stMessage->aData[3] = (uint8_t)stSensorData->stXyzData.iYAccel;

    stMessage->aData[4] = (uint8_t)(stSensorData->stXyzData.iZAccel >> 8);
    stMessage->aData[5] = (uint8_t)stSensorData->stXyzData.iZAccel;

    stMessage->bWriting = 0;

    return;
}

// Timer 3 - Sensor Input, CAN I/O, Heartbeat Monitor
void acmIsrT3(void)
{
    // adc1Convert();

    // Sensor Input
    // acmSensorInputSwExt();			// read switch, not relevent

    // CAN Input
    // acmCanInputBatt();
    acmCanInputDuplicateHb();			// data operation
    acmCanInputHeartbeat();
    // acmCanInputRpm();
    acmCanInputShutdown();				// data operation
    acmCanInputSwLimit();				// read back wing position, update stAcmSensor data. but this data is faked. refer to acwT3.
    // acmCanInputWheelspeed(); 

    // CAN Output
    acmCanOutputServoLimits();			// put actual data into tx array: 1c08, 1c0a
    acmCanOutputServoPosition();		// put max/min data into tx array: 1c00, 1c02, 1c04, 1c06
    acmCanOutputShutdown();				// put data in 0x200
    acmCanOutputState();				// feed 1c10, but never transmitted
    acmCanOutputSwExt();				// feed 1a00, use the data from acmCanInputSwLimit().
    acmCanOutputXyz();					// feed 1a00, while we dont have sensor data actually. so they could be anything here

    // Heartbeat Monitoring
    heartbeat_mwDecrement(ACM_HEARTBEAT_TIMER_ACW_LF);
    // heartbeat_mwDecrement(ACM_HEARTBEAT_TIMER_ACW_RF);
    // heartbeat_mwDecrement(ACM_HEARTBEAT_TIMER_ACW_LR);
    heartbeat_mwDecrement(ACM_HEARTBEAT_TIMER_ACW_RR);
    
    return;
}

uint16_t T3_out_RR(void)
{
	return aHeartbeatTimer[ACM_HEARTBEAT_TIMER_ACW_RR];
}

uint16_t T3_out_LF(void)
{
	return aHeartbeatTimer[ACM_HEARTBEAT_TIMER_ACW_LF];
}