#include "stdint.h"
#include "acm.h"
#include "acmIsrT4.h"

uint8_t Percent[4];
void T4_in(uint8_t *T4_buffer)
{
	int i;
	for(i=0;i<4;i++)
	{
		Percent[i] = T4_buffer[i];
	}
}
// Initialize a servo data struct.
void servo_mwInit(ServoData * stServoData)
{
    stServoData->bCalibrationFailed = 0;
    stServoData->bEnabled = 1;
    stServoData->bInvert = 0;
    stServoData->bLimitViolation = 0;
    stServoData->bMoving = 0;
    stServoData->bPowered = 1;

    stServoData->iTickCount = 0;
    stServoData->iTickMax = 0;
    stServoData->iTickMin = 0;
    stServoData->iTickSinceLastMove = 0;

    stServoData->iPosCurrent = 0;
    stServoData->iPosMax = 65535;
    stServoData->iPosMin = 0;
    stServoData->iPosNeutral = 0;
    stServoData->iPosPrevious = 0;
    stServoData->iPosTarget = 0;

    stServoData->iServo = 0;

    return;
}

// Initialize servo-specific hardware and software.
void acmServoInit(void)
{
    uint8_t iIndex = 0;

    for(iIndex = 0; iIndex < ACM_SERVO_COUNT; iIndex++)
    {
        servo_mwInit(&aAcmServo[iIndex]);

        aAcmServo[iIndex].iServo = iIndex;
        aAcmServo[iIndex].iPosMax = ACM_SERVO_POSITION_MAXIMUM;
        aAcmServo[iIndex].iPosMin = ACM_SERVO_POSITION_MINIMUM;

        aAcmServo[iIndex].iPosNeutral = ACM_SERVO_POSITION_MAXIMUM;
        aAcmServo[iIndex].iPosNeutral -= ACM_SERVO_POSITION_MINIMUM;
        aAcmServo[iIndex].iPosNeutral /= 2;
        aAcmServo[iIndex].iPosNeutral += ACM_SERVO_POSITION_MINIMUM;
    }

    // Disable missing servos.
    aAcmServo[ACM_SERVO_LFL].bEnabled = ACM_SERVO_ENABLE_LFL;
    aAcmServo[ACM_SERVO_LFU].bEnabled = ACM_SERVO_ENABLE_LFU;

    aAcmServo[ACM_SERVO_RFL].bEnabled = ACM_SERVO_ENABLE_RFL;
    aAcmServo[ACM_SERVO_RFU].bEnabled = ACM_SERVO_ENABLE_RFU;

    aAcmServo[ACM_SERVO_LRL].bEnabled = ACM_SERVO_ENABLE_LRL;
    aAcmServo[ACM_SERVO_LRU].bEnabled = ACM_SERVO_ENABLE_LRU;

    aAcmServo[ACM_SERVO_RRL].bEnabled = ACM_SERVO_ENABLE_RRL;
    aAcmServo[ACM_SERVO_RRU].bEnabled = ACM_SERVO_ENABLE_RRU;

    // Set waveform inversion bits.
    aAcmServo[ACM_SERVO_LFL].bInvert = ACM_SERVO_INVERT_LFL;
    aAcmServo[ACM_SERVO_LFU].bInvert = ACM_SERVO_INVERT_LFU;

    aAcmServo[ACM_SERVO_RFL].bInvert = ACM_SERVO_INVERT_RFL;
    aAcmServo[ACM_SERVO_RFU].bInvert = ACM_SERVO_INVERT_RFU;

    aAcmServo[ACM_SERVO_LRL].bInvert = ACM_SERVO_INVERT_LRL;
    aAcmServo[ACM_SERVO_LRU].bInvert = ACM_SERVO_INVERT_LRU;

    aAcmServo[ACM_SERVO_RRL].bInvert = ACM_SERVO_INVERT_RRL;
    aAcmServo[ACM_SERVO_RRU].bInvert = ACM_SERVO_INVERT_RRU;

    // Set servo maximum tick limits.
    aAcmServo[ACM_SERVO_LFL].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_LFL;
    aAcmServo[ACM_SERVO_LFU].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_LFU;

    aAcmServo[ACM_SERVO_RFL].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_RFL;
    aAcmServo[ACM_SERVO_RFU].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_RFU;

    aAcmServo[ACM_SERVO_LRL].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_LRL;
    aAcmServo[ACM_SERVO_LRU].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_LRU;

    aAcmServo[ACM_SERVO_RRL].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_RRL;
    aAcmServo[ACM_SERVO_RRU].iTickMax = ACM_CALIBRATE_TICK_LIMIT_MAX_RRU;

    // Set servo minimum tick limits.
    aAcmServo[ACM_SERVO_LFL].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_LFL;
    aAcmServo[ACM_SERVO_LFU].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_LFU;

    aAcmServo[ACM_SERVO_RFL].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_RFL;
    aAcmServo[ACM_SERVO_RFU].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_RFU;

    aAcmServo[ACM_SERVO_LRL].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_LRL;
    aAcmServo[ACM_SERVO_LRU].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_LRU;

    aAcmServo[ACM_SERVO_RRL].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_RRL;
    aAcmServo[ACM_SERVO_RRU].iTickMin = ACM_CALIBRATE_TICK_LIMIT_MIN_RRU;

    return;
}


void acmServoPowerOff(void)
{
    AcmSensorData * stServoData = acmSensorRet();
    ServoData * aServoData = acmServoRet();

    uint8_t bPowered = 0;
    uint8_t iServo = 0;

    for(iServo = 0; iServo < ACM_SERVO_COUNT; iServo++)
    {
        bPowered = 0;

        if(aServoData[iServo].bEnabled != 1)
            continue;

        if(aServoData[iServo].iTickSinceLastMove <  ACM_SERVO_POWER_OFF_TICK_COUNT)
            bPowered = 1;

        // Check for open limit switch.
        if( ((stServoData->iSwLimit >> iServo) & 0x01) == 0)
            bPowered = 1;

        aServoData[iServo].bPowered = bPowered;
    }

    return;
}

// Find lookup table column while the accelerometer X axis is pointed toward
// the front of the car.
uint8_t acmLookupColumnXFront(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iColumnIndex = ACM_TABLE_COLUMN_CENTER;
    int8_t iColumnDelta = stXyzData->iYAccel / 100;

    // Find the absolute value of the column offset.
    if(iColumnDelta < 0)
        iColumnDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iYAccel > 0)
    {
        // The car is turning right, so the table index will move right.
        if(stXyzData->iYAccel <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex += iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MAX;
    }
    else
    {
        // The car is turning left, so the table index will move left.
        if(stXyzData->iYAccel * -1 <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex -= iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MIN;
    }

    return iColumnIndex;
}


// Find lookup table column while the accelerometer X axis is pointed toward
// the left side of the car.
uint8_t acmLookupColumnXLeft(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iColumnIndex = ACM_TABLE_COLUMN_CENTER;		//20
    int8_t iColumnDelta = stXyzData->iXAccel / 100;		//-121

    // Find the absolute value of the column offset.
    if(iColumnDelta < 0)
        iColumnDelta *= -1;				//121

    // Perform the index translation.
    if(stXyzData->iXAccel > 0)
    {
        // The car is turning right, so the table index will move right.
        if(stXyzData->iXAccel <= ACM_TABLE_LIMIT_LATERAL)		// 1900
            iColumnIndex += iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MAX;
    }
    else
    {
        // The car is turning left, so the table index will move left.
        if(stXyzData->iXAccel * -1 <= ACM_TABLE_LIMIT_LATERAL)		//12167,1900	
            iColumnIndex -= iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MIN;			//0
    }

    return iColumnIndex;
}


// Find lookup table column while the accelerometer X axis is pointed toward
// the rear of the car.
uint8_t acmLookupColumnXRear(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iColumnIndex = ACM_TABLE_COLUMN_CENTER;
    int8_t iColumnDelta = stXyzData->iYAccel / 100;

    // Find the absolute value of the column offset.
    if(iColumnDelta < 0)
        iColumnDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iYAccel > 0)
    {
        // The car is turning left, so the table index will move left.
        if(stXyzData->iYAccel <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex -= iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MIN;
    }
    else
    {
        // The car is turning right, so the table index will move right.
        if(stXyzData->iYAccel * -1 <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex += iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MAX;
    }

    return iColumnIndex;
}


// Find lookup table column while the accelerometer X axis is pointed toward
// the right side of the car.
uint8_t acmLookupColumnXRight(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iColumnIndex = ACM_TABLE_COLUMN_CENTER;
    int8_t iColumnDelta = stXyzData->iXAccel / 100;

    // Find the absolute value of the column offset.
    if(iColumnDelta < 0)
        iColumnDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iXAccel > 0)
    {
        // The car is turning left, so the table index will move left.
        if(stXyzData->iXAccel <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex -= iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MIN;
    }
    else
    {
        // The car is turning right, so the table index will move right.
        if(stXyzData->iXAccel * -1 <= ACM_TABLE_LIMIT_LATERAL)
            iColumnIndex += iColumnDelta;
        else
            iColumnIndex = ACM_TABLE_COLUMN_MAX;
    }

    return iColumnIndex;
}


// Find the column index to use when looking up data in the position table.
uint8_t acmLookupColumn(XyzData * stXyzData)
{
    uint8_t iReturn = 0;

    switch(ACM_DIRECTION_X)
    {
        // Positive X axis toward front of the car
        case 0:
        default:
        {
            iReturn = acmLookupColumnXFront(stXyzData);
            break;
        }

        // Positive X axis toward right side of the car
        case 1:
        {
            iReturn = acmLookupColumnXRight(stXyzData);
            break;
        }

        // Positive X axis toward rear of the car
        case 2:
        {
            iReturn = acmLookupColumnXRear(stXyzData);
            break;
        }

        // Positive X axis toward left side of the car
        case 3:
        {
            iReturn = acmLookupColumnXLeft(stXyzData);
            break;
        }
    }

    return iReturn;
}


// Find lookup table row while the accelerometer X axis is pointed toward
// the front of the car.
uint8_t acmLookupRowXFront(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iRowIndex = ACM_TABLE_ROW_CENTER;
    int8_t iRowDelta = stXyzData->iXAccel / 100;

    // Find the absolute value of the row offset.
    if(iRowDelta < 0)
        iRowDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iXAccel > 0)
    {
        // The car is slowing down, so the table index will move down.
        if(stXyzData->iXAccel <= ACM_TABLE_LIMIT_LONGITUDINAL_DECEL)
            iRowIndex += iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MAX;
    }
    else
    {
        // The car is speeding up, so the table index will move up.
        if(stXyzData->iXAccel * -1 <= ACM_TABLE_LIMIT_LONGITUDINAL_ACCEL)
            iRowIndex -= iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MIN;
    }

    return iRowIndex;
}


// Find lookup table row while the accelerometer X axis is pointed toward
// the left side of the car.
uint8_t acmLookupRowXLeft(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iRowIndex = ACM_TABLE_ROW_CENTER;			//13
    int8_t iRowDelta = stXyzData->iYAccel / 100;		//75

    // Find the absolute value of the row offset.
    if(iRowDelta < 0)
        iRowDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iYAccel > 0)
    {
        // The car is speeding up, so the table index will move up.
        if(stXyzData->iYAccel <= ACM_TABLE_LIMIT_LONGITUDINAL_ACCEL)		//7583>=1200
            iRowIndex -= iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MAX;									//33
    }
    else
    {
        // The car is slowing down, so the table index will move down.
        if(stXyzData->iYAccel * -1 <= ACM_TABLE_LIMIT_LONGITUDINAL_DECEL)
            iRowIndex += iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MIN;
    }

    return iRowIndex;
}


// Find lookup table row while the accelerometer X axis is pointed toward
// the rear of the car.
uint8_t acmLookupRowXRear(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iRowIndex = ACM_TABLE_ROW_CENTER;
    int8_t iRowDelta = stXyzData->iXAccel / 100;

    // Find the absolute value of the row offset.
    if(iRowDelta < 0)
        iRowDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iXAccel > 0)
    {
        // The car is speeding up, so the table index will move up.
        if(stXyzData->iXAccel <= ACM_TABLE_LIMIT_LONGITUDINAL_ACCEL)
            iRowIndex -= iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MIN;
    }
    else
    {
        // The car is slowing down, so the table index will move down.
        if(stXyzData->iXAccel * -1 <= ACM_TABLE_LIMIT_LONGITUDINAL_DECEL)
            iRowIndex += iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MAX;
    }

    return iRowIndex;
}


// Find lookup table row while the accelerometer X axis is pointed toward
// the right side of the car.
uint8_t acmLookupRowXRight(XyzData * stXyzData)
{
    // A G map's origin is at point 0,0.  This must be translated to an index
    // value that can be used with the table.
    uint8_t iRowIndex = ACM_TABLE_ROW_CENTER;
    int8_t iRowDelta = stXyzData->iYAccel / 100;

    // Find the absolute value of the row offset.
    if(iRowDelta < 0)
        iRowDelta *= -1;

    // Perform the index translation.
    if(stXyzData->iYAccel > 0)
    {
        // The car is slowing down, so the table index will move down.
        if(stXyzData->iYAccel <= ACM_TABLE_LIMIT_LONGITUDINAL_DECEL)
            iRowIndex += iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MAX;
    }
    else
    {
        // The car is speeding up, so the table index will move up.
        if(stXyzData->iYAccel * -1 <= ACM_TABLE_LIMIT_LONGITUDINAL_ACCEL)
            iRowIndex -= iRowDelta;
        else
            iRowIndex = ACM_TABLE_ROW_MIN;
    }

    return iRowIndex;
}


// Find the row index to use when looking up data in the position table.
uint8_t acmLookupRow(XyzData * stXyzData)
{
    uint8_t iReturn = 0;

    switch(ACM_DIRECTION_X)				//3
    {
        // Positive X axis toward front of the car
        case 0:
        default:
        {
            iReturn = acmLookupRowXFront(stXyzData);
            break;
        }

        // Positive X axis toward right side of the car
        case 1:
        {
            iReturn = acmLookupRowXRight(stXyzData);
            break;
        }

        // Positive X axis toward rear of the car
        case 2:
        {
            iReturn = acmLookupRowXRear(stXyzData);
            break;
        }

        // Positive X axis toward left side of the car
        case 3:
        {
            iReturn = acmLookupRowXLeft(stXyzData);
            break;
        }
    }

    return iReturn;
}

// Find the wing closed percentage.
uint8_t acmLookupWing(ACM_Wing eACMWing, XyzData * stXyzData, uint8_t iLookupTable)
{
    uint8_t iColumn = acmLookupColumn(stXyzData);		//0
    uint8_t iRow = acmLookupRow(stXyzData);				//33

    switch(eACMWing)
    {
        case ACM_Wing_Left_Front:
        {
            return aAeroLeftFront0[iRow][iColumn];
            break;
        }

        case ACM_Wing_Left_Rear:
        {
            return aAeroLeftRear0[iRow][iColumn];
            break;
        }

        case ACM_Wing_Right_Front:
        {
            return aAeroRightFront0[iRow][iColumn];
            break;
        }

        case ACM_Wing_Right_Rear:
        {
            return aAeroRightRear0[iRow][iColumn];
            break;
        }
    }

    return 0;
}

// Active Mode - position the wings by comparing accelerometer input to a
// lookup table.
void acmStateDoActive(uint8_t iLookupTable)
{
    AcmSensorData * stAcmSensor = acmSensorRet();
    ServoData * aServoData = acmServoRet();

    XyzData stXyzData;

    // Check the override switch.
    // if(stAcmSensor->iSwExt & (1 << ACM_BIT_SW_OVERRIDE))
    // {
        // acmStateDoClose();
        // return;
    // }

    // Read wing positions from the lookup table.
    // stXyzData.iXAccel = (int16_t)stAcmSensor->stXyzData.iXAccel;		//-12167
    // stXyzData.iYAccel = (int16_t)stAcmSensor->stXyzData.iYAccel;		//7583

    // uint8_t iLFPercent = acmLookupWing(ACM_Wing_Left_Front, &stXyzData, iLookupTable);		//0,
    // uint8_t iRFPercent = acmLookupWing(ACM_Wing_Right_Front, &stXyzData, iLookupTable);
    // uint8_t iLRPercent = acmLookupWing(ACM_Wing_Left_Rear, &stXyzData, iLookupTable);
    // uint8_t iRRPercent = acmLookupWing(ACM_Wing_Right_Rear, &stXyzData, iLookupTable);
	
    uint8_t iLFPercent = Percent[0];
    uint8_t iRFPercent = Percent[1];
    uint8_t iLRPercent = Percent[2];
    uint8_t iRRPercent = Percent[3];

    // Calculate servo movement range deltas.  LFL = left-front lower element.
    uint32_t iLFLDelta = aServoData[ACM_SERVO_LFL].iPosMax - aServoData[ACM_SERVO_LFL].iPosMin;		//670
    uint32_t iLFUDelta = aServoData[ACM_SERVO_LFU].iPosMax - aServoData[ACM_SERVO_LFU].iPosMin;

    uint32_t iRFLDelta = aServoData[ACM_SERVO_RFL].iPosMax - aServoData[ACM_SERVO_RFL].iPosMin;
    uint32_t iRFUDelta = aServoData[ACM_SERVO_RFU].iPosMax - aServoData[ACM_SERVO_RFU].iPosMin;

    uint32_t iLRLDelta = aServoData[ACM_SERVO_LRL].iPosMax - aServoData[ACM_SERVO_LRL].iPosMin;
    uint32_t iLRUDelta = aServoData[ACM_SERVO_LRU].iPosMax - aServoData[ACM_SERVO_LRU].iPosMin;

    uint32_t iRRLDelta = aServoData[ACM_SERVO_RRL].iPosMax - aServoData[ACM_SERVO_RRL].iPosMin;
    uint32_t iRRUDelta = aServoData[ACM_SERVO_RRU].iPosMax - aServoData[ACM_SERVO_RRU].iPosMin;

    // Calculate servo duty cycles.
	uint16_t iLFLDuty = aServoData[ACM_SERVO_LFL].iPosMin + ((iLFLDelta * iLFPercent) / 100);			//0
    uint16_t iLFUDuty = aServoData[ACM_SERVO_LFU].iPosMin + ((iLFUDelta * iLFPercent) / 100);			//1
	
    uint16_t iRFLDuty = aServoData[ACM_SERVO_RFL].iPosMax - ((iRFLDelta * iRFPercent) / 100);			//2
    uint16_t iRFUDuty = aServoData[ACM_SERVO_RFU].iPosMax - ((iRFUDelta * iRFPercent) / 100);			//3

    uint16_t iLRLDuty = aServoData[ACM_SERVO_LRL].iPosMin + ((iLRLDelta * iLRPercent) / 100);			//4
    uint16_t iLRUDuty = aServoData[ACM_SERVO_LRU].iPosMin + ((iLRUDelta * iLRPercent) / 100);			//5

    uint16_t iRRLDuty = aServoData[ACM_SERVO_RRL].iPosMax - ((iRRLDelta * iRRPercent) / 100);			//6
    uint16_t iRRUDuty = aServoData[ACM_SERVO_RRU].iPosMax - ((iRRUDelta * iRRPercent) / 100);			//7

    // Position wings
    servo_mwSetPosition(&aServoData[ACM_SERVO_LFL], iLFLDuty);		// 1c08
    servo_mwSetPosition(&aServoData[ACM_SERVO_LFU], iLFUDuty);		// 1c08

    servo_mwSetPosition(&aServoData[ACM_SERVO_RFL], iRFLDuty);		// 1c08
    servo_mwSetPosition(&aServoData[ACM_SERVO_RFU], iRFUDuty);		// 1c08

    servo_mwSetPosition(&aServoData[ACM_SERVO_LRL], iLRLDuty);		// 1c0a
    servo_mwSetPosition(&aServoData[ACM_SERVO_LRU], iLRUDuty);		// 1c0a

    servo_mwSetPosition(&aServoData[ACM_SERVO_RRL], iRRLDuty);		// 1c0a
    servo_mwSetPosition(&aServoData[ACM_SERVO_RRU], iRRUDuty);		// 1c0a

    acmServoPowerOff();

    return;
}

// Static Mode - set the wings to the fully closed position.
void acmStateDoClose(void)
{
	ServoData * aServoData = acmServoRet();

    servo_mwSetPosition(&aServoData[ACM_SERVO_LFL], aServoData[ACM_SERVO_LFL].iPosMin);
    servo_mwSetPosition(&aServoData[ACM_SERVO_LFU], aServoData[ACM_SERVO_LFU].iPosMin);

    servo_mwSetPosition(&aServoData[ACM_SERVO_RFL], aServoData[ACM_SERVO_RFL].iPosMax);
    servo_mwSetPosition(&aServoData[ACM_SERVO_RFU], aServoData[ACM_SERVO_RFU].iPosMax);

    servo_mwSetPosition(&aServoData[ACM_SERVO_LRL], aServoData[ACM_SERVO_LRL].iPosMin);
    servo_mwSetPosition(&aServoData[ACM_SERVO_LRU], aServoData[ACM_SERVO_LRU].iPosMin);

    servo_mwSetPosition(&aServoData[ACM_SERVO_RRL], aServoData[ACM_SERVO_RRL].iPosMax);
    servo_mwSetPosition(&aServoData[ACM_SERVO_RRU], aServoData[ACM_SERVO_RRU].iPosMax);

    acmServoPowerOff();

    return;
}

// Call the appropriate state handler function.
void acmStateSet(void)
{
    switch(eACMState)
    {
        case ACM_State_Active0:
        {
            acmStateDoActive(0);
            break;
        }

        case ACM_State_Active1:
        {
            acmStateDoActive(1);
            break;
        }

        case ACM_State_Close:
        {
            acmStateDoClose();
            break;
        }
	
/*        case ACM_State_Demo:
        {
            acmStateDoDemo();
            break;
        }

        case ACM_State_Lock:
        {
            acmStateDoLock();
            break;
        }

        case ACM_State_Manual:
        {
            acmStateDoManual();
            break;
        }

        case ACM_State_Open:
        default:
        {
            acmStateDoOpen();
            break;
        }*/
    }

    return;
}

// Timer 4 - AAC State Machine Timer
void acmIsrT4(void)
{
    acmStateSet();
    return;
}

uint16_t T4_out(int n)
{
	// ServoData * aServoData = acmServoRet();
	return aAcmServo[n].iPosTarget;//aServoData[a].iPosTarget;
	// return Percent[n];
}

