// Timer 1 - TTCAN1 Master Timer, this version is
// for stateflow, no mex function involved.
// #include "stdint.h"
#include "acm.h"
#include "acwIsrT1.h"

void acwCanInitTableTxRR_1(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // ACW RR Heartbeat
    canInitMessage(&stMessage);
    stMessage.iId = 0x468;

    ctSetMessage_acw_1(&stMessage, CanTable_Transmit, 0);

    // *************************************************************************
    // Active Aero Control Limit Switches - Right Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e06;

    ctSetMessage_acw_1(&stMessage, CanTable_Transmit, 1);

    return;
}

void acwCanInitTableTxRR_2(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // ACW RR Heartbeat
    canInitMessage(&stMessage);
    stMessage.iId = 0x468;

    ctSetMessage_acw_2(&stMessage, CanTable_Transmit, 0);

    // *************************************************************************
    // Active Aero Control Limit Switches - Right Rear Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e06;

    ctSetMessage_acw_2(&stMessage, CanTable_Transmit, 1);

    return;
}

void acwCanInitTableTxLF_1(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // ACW LF Heartbeat
    canInitMessage(&stMessage);
    stMessage.iId = 0x462;

    ctSetMessage_acw_1(&stMessage, CanTable_Transmit, 0);

    // *************************************************************************
    // Active Aero Control Limit Switches - Left Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e00;

    ctSetMessage_acw_1(&stMessage, CanTable_Transmit, 1);

    return;
}

void acwCanInitTableTxLF_2(void)
{
    CanMessage stMessage;

    // *************************************************************************
    // ACW LF Heartbeat
    canInitMessage(&stMessage);
    stMessage.iId = 0x462;

    ctSetMessage_acw_2(&stMessage, CanTable_Transmit, 0);

    // *************************************************************************
    // Active Aero Control Limit Switches - Left Front Wing
    canInitMessage(&stMessage);
    stMessage.iId = 0x1e00;

    ctSetMessage_acw_2(&stMessage, CanTable_Transmit, 1);

    return;
}

void acwCanInitTableTx_1(void)
{
    if(ACW_CAN_ADDRESS_1 == ACW_LF_CAN_ADDRESS)
        acwCanInitTableTxLF_1();

    #if(ACW_CAN_ADDRESS_1 == ACW_LR_CAN_ADDRESS)
        acwCanInitTableTxLR_1();
    #endif

    #if(ACW_CAN_ADDRESS_1 == ACW_RF_CAN_ADDRESS)
        acwCanInitTableTxRF_1();
    #endif

    if(ACW_CAN_ADDRESS_1 == ACW_RR_CAN_ADDRESS)
        acwCanInitTableTxRR_1();

    return;
}

void acwCanInitTableTx_2(void)
{
    if(ACW_CAN_ADDRESS_2 == ACW_LF_CAN_ADDRESS)
        acwCanInitTableTxLF_2();

    #if(ACW_CAN_ADDRESS_2 == ACW_LR_CAN_ADDRESS)
        acwCanInitTableTxLR_2();
    #endif

    #if(ACW_CAN_ADDRESS_2 == ACW_RF_CAN_ADDRESS)
        acwCanInitTableTxRF_2();
    #endif

    if(ACW_CAN_ADDRESS_2 == ACW_RR_CAN_ADDRESS)
        acwCanInitTableTxRR_2();

    return;
}

void directTxMessage_acw_1(CanMessage * stCanMessage)
{
    static uint16_t aCAN1BufferWord_acw_1[8];
    uint8_t iBufferIndex = 0;   // Index for aCAN1BufferWord_acw_1
    uint8_t iDataIndex = 0;     // Index for aData within stCanMessage    
	#ifdef ENABLE_MODULE_ECAN_1

    // if(stCanMessage == NULL)
        // return;

    // Validate the Data Length Code.
    if(stCanMessage->iDlc > 8)
        stCanMessage->iDlc = 0;

    // Initialize the buffer word array.
    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
        aCAN1BufferWord_acw_1[iBufferIndex] = 0;
	
	for(iDataIndex = 0; iDataIndex < 8; iDataIndex++)
		aCAN1BufferWord_acw_1[iDataIndex] = stCanMessage->aData[iDataIndex];
	
    // ********************************************************************
    // Copy the CAN buffer words into the DMA buffer
    // Remember, DMA buffer 0 has been setup as an ECAN TX buffer.

    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
		aCAN1Buffer_acw_1[0][iBufferIndex] = aCAN1BufferWord_acw_1[iBufferIndex] & 0xff;
	#endif
	return;
}

void directTxMessage_acw_2(CanMessage * stCanMessage)
{
    static uint16_t aCAN1BufferWord_acw_2[8];
    uint8_t iBufferIndex = 0;   // Index for aCAN1BufferWord_acw_2
    uint8_t iDataIndex = 0;     // Index for aData within stCanMessage    
	#ifdef ENABLE_MODULE_ECAN_1

    // if(stCanMessage == NULL)
        // return;

    // Validate the Data Length Code.
    if(stCanMessage->iDlc > 8)
        stCanMessage->iDlc = 0;

    // Initialize the buffer word array.
    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
        aCAN1BufferWord_acw_2[iBufferIndex] = 0;
	
	for(iDataIndex = 0; iDataIndex < 8; iDataIndex++)
		aCAN1BufferWord_acw_2[iDataIndex] = stCanMessage->aData[iDataIndex];
	
    // ********************************************************************
    // Copy the CAN buffer words into the DMA buffer
    // Remember, DMA buffer 0 has been setup as an ECAN TX buffer.

    for(iBufferIndex = 0; iBufferIndex < 8; iBufferIndex++)
		aCAN1Buffer_acw_2[0][iBufferIndex] = aCAN1BufferWord_acw_2[iBufferIndex] & 0xff;
	#endif
	return;
}

void acwTimer1Stop_1()
{
	acwtimer_1 = 0;
}

void acwTimer1Stop_2()
{
	acwtimer_2 = 0;
}

void acwIsrT1_1(void)
{
    if(acwtimer_1 == 1)			// simulating the behaviour of turn on/off this timer.		
	{
		uint32_t iCanId = 0x000;

		uint16_t iCanAddress;
		uint8_t * iTTCan1Cycle;
		uint8_t * iTTCan1Window;

		CanMessage * stMessage;

		// Abort previous window transmission.
		can1AbortTx_acw_1(0);

		iCanAddress = acwCanGetAddress_1();			// 0x468
		iTTCan1Cycle = acwCan1GetCycle_1();
		iTTCan1Window = acwCan1GetWindow_1();

		// Retrieve the message ID from the TTCAN schedule
		iCanId = e15aerTtCanGetId(iCanAddress, *iTTCan1Cycle, *iTTCan1Window);
		stMessage = ctGetMessageById_acw_1(CanTable_Transmit, iCanId);
		
		// *************************************************************************
		// Transmission time!

		if(iCanId != 0x00)
		{
			// acwCan1RxLedOn();
			// can1TxMessage(stMessage);
			directTxMessage_acw_1(stMessage);
			// T1_Data_Out_acw(iCanId, *iTTCan1Cycle, *iTTCan1Window);
		}
		T1_Data_Out_acw_1(iCanId, *iTTCan1Cycle, *iTTCan1Window);
		// else
			// acwCan1RxLedOff();

		// *************************************************************************
		// TTCAN update time!    

		// Increment the window
		(*iTTCan1Window)++;

		// If this is the last window in the cycle, stop timer 1 and reset the
		// TTCAN window index.
		if((*iTTCan1Window) >= E15AER_TTCAN_SCHEDULE_WINDOWS)		// 10
		{
			acwTimer1Stop_1();
			(*iTTCan1Window) = 0;

			// acwCan1RxLedOff();
		}
	}
    return;
}

void acwIsrT1_2(void)
{
    if(acwtimer_2 == 1)			// simulating the behaviour of turn on/off this timer.		
	{
		uint32_t iCanId = 0x000;

		uint16_t iCanAddress;
		uint8_t * iTTCan1Cycle;
		uint8_t * iTTCan1Window;

		CanMessage * stMessage;

		// Abort previous window transmission.
		can1AbortTx_acw_2(0);

		iCanAddress = acwCanGetAddress_2();			// 0x468
		iTTCan1Cycle = acwCan1GetCycle_2();
		iTTCan1Window = acwCan1GetWindow_2();

		// Retrieve the message ID from the TTCAN schedule
		iCanId = e15aerTtCanGetId(iCanAddress, *iTTCan1Cycle, *iTTCan1Window);
		stMessage = ctGetMessageById_acw_2(CanTable_Transmit, iCanId);
		
		// *************************************************************************
		// Transmission time!

		if(iCanId != 0x00)
		{
			// acwCan1RxLedOn();
			// can1TxMessage(stMessage);
			directTxMessage_acw_2(stMessage);
			// T1_Data_Out_acw(iCanId, *iTTCan1Cycle, *iTTCan1Window);
		}
		T1_Data_Out_acw_2(iCanId, *iTTCan1Cycle, *iTTCan1Window);
		// else
			// acwCan1RxLedOff();

		// *************************************************************************
		// TTCAN update time!    

		// Increment the window
		(*iTTCan1Window)++;

		// If this is the last window in the cycle, stop timer 1 and reset the
		// TTCAN window index.
		if((*iTTCan1Window) >= E15AER_TTCAN_SCHEDULE_WINDOWS)		// 10
		{
			acwTimer1Stop_2();
			(*iTTCan1Window) = 0;

			// acwCan1RxLedOff();
		}
		
	}
    return;
}

void T1_in_acw(int acw_can_addr, int instance)
{
	if(instance == 1)
		ACW_CAN_ADDRESS_1 = acw_can_addr;
	if(instance == 2)
		ACW_CAN_ADDRESS_2 = acw_can_addr;
}
	
void T1_Data_Out_acw_1(uint32_t canid, uint8_t cycle, uint8_t window)
{
	int i;
	for(i=0;i<11;i++)
		TIMER1_OUT_ACW_1[i] = 0;
	for(i=0;i<8;i++)
		TIMER1_OUT_ACW_1[i] = aCAN1Buffer_acw_1[0][i]&0x00ffff;
	TIMER1_OUT_ACW_1[8] = canid;
	TIMER1_OUT_ACW_1[9] = cycle;
	TIMER1_OUT_ACW_1[10] = window;
}

void T1_Data_Out_acw_2(uint32_t canid, uint8_t cycle, uint8_t window)
{
	int i;
	for(i=0;i<11;i++)
		TIMER1_OUT_ACW_2[i] = 0;
	for(i=0;i<8;i++)
		TIMER1_OUT_ACW_2[i] = aCAN1Buffer_acw_2[0][i]&0x00ffff;
	TIMER1_OUT_ACW_2[8] = canid;
	TIMER1_OUT_ACW_2[9] = cycle;
	TIMER1_OUT_ACW_2[10] = window;
}

uint16_t T1_out_acw(uint8_t n,int instance)
{
	if(instance == 1)
		return TIMER1_OUT_ACW_1[n];
	if(instance == 2)
		return TIMER1_OUT_ACW_2[n];
	return 0;
}
